# THIS FILE HAS BEEN AUTOMATICALLY GENERATED.
# CHANGES TO THIS FILE ARE FUTILE AS IT WILL BE OVERWRITTEN
# WHEN THE DEVTOOLS PROTOCOL CHANGES.  IF THERE ANY BUGS
# OR YOU WISH TO CHANGE HOW THE FILES ARE GENERATED PLEASE
# REFER TO: https://github.com/symonk/python-cdp OR YOUR
# OWN FORK.  REFERENCE THE `generate.py` FILE FOR CONTEXT
# AND INSTRUCTIONS.
# Chrome Devtools Protocol Domain Mapped to: `Preload`.
# Url for domain: https://chromedevtools.github.io/devtools-protocol/tot/Preload/

from __future__ import annotations

import enum
import typing
from dataclasses import dataclass

from . import dom
from . import network
from . import page
from .utils import memoize_event


class RuleSetId(str):
    """Unique id."""

    def to_json(self) -> RuleSetId:
        return self

    @classmethod
    def from_json(cls, value: str) -> RuleSetId:
        return cls(value)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(({super().__repr__()}))"


@dataclass
class RuleSet:
    """Corresponds to SpeculationRuleSet."""

    # Description is missing from the devtools protocol document. # noqa
    id: RuleSetId
    # Identifies a document which the rule set is associated with. # noqa
    loader_id: network.LoaderId
    # Source text of JSON representing the rule set. If it comes from <script>tag, it is the textContent of the node. Note that it is a JSON for valid case.See also: - https://wicg.github.io/nav-speculation/speculation-rules.html -https://github.com/WICG/nav-speculation/blob/main/triggers.md # noqa
    source_text: str
    # Error information `errorMessage` is null iff `errorType` is null. # noqa
    error_type: typing.Optional[RuleSetErrorType]
    # TODO(https://crbug.com/1425354): Replace this property with structurederror. # noqa
    error_message: typing.Optional[str]


class RuleSetErrorType(str, enum.Enum):
    """Description is missing from the devtools protocol document."""

    SOURCE_IS_NOT_JSON_OBJECT = "source_is_not_json_object"
    INVALID_RULES_SKIPPED = "invalid_rules_skipped"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


class SpeculationAction(str, enum.Enum):
    """The type of preloading attempted.

    It corresponds to mojom::SpeculationAction (although PrefetchWithSubresources is omitted as it isn't being used by
    clients).
    """

    PREFETCH = "prefetch"
    PRERENDER = "prerender"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


class SpeculationTargetHint(str, enum.Enum):
    """Corresponds to mojom::SpeculationTargetHint.

    See https://github.com/WICG/nav-speculation/blob/main/triggers.md#window-name-targeting-hints
    """

    BLANK = "blank"
    SELF = "self"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


@dataclass
class PreloadingAttemptKey:
    """A key that identifies a preloading attempt.

    The url used is the url specified by the trigger (i.e. the initial URL), and not the final url that is navigated to.
    For example, prerendering allows same-origin main frame navigations during the attempt, but the attempt is still
    keyed with the initial URL.
    """

    # Description is missing from the devtools protocol document. # noqa
    loader_id: network.LoaderId
    # Description is missing from the devtools protocol document. # noqa
    action: SpeculationAction
    # Description is missing from the devtools protocol document. # noqa
    url: str
    # Description is missing from the devtools protocol document. # noqa
    target_hint: typing.Optional[SpeculationTargetHint]


@dataclass
class PreloadingAttemptSource:
    """Lists sources for a preloading attempt, specifically the ids of rule sets that had a speculation rule that
    triggered the attempt, and the BackendNodeIds of <a href> or <area href> elements that triggered the attempt (in the
    case of attempts triggered by a document rule).

    It is possible for mulitple rule sets and links to trigger a single attempt.
    """

    # Description is missing from the devtools protocol document. # noqa
    key: PreloadingAttemptKey
    # Description is missing from the devtools protocol document. # noqa
    rule_set_ids: RuleSetId
    # Description is missing from the devtools protocol document. # noqa
    node_ids: dom.BackendNodeId


class PrerenderFinalStatus(str, enum.Enum):
    """List of FinalStatus reasons for Prerender2."""

    ACTIVATED = "activated"
    DESTROYED = "destroyed"
    LOW_END_DEVICE = "low_end_device"
    INVALID_SCHEME_REDIRECT = "invalid_scheme_redirect"
    INVALID_SCHEME_NAVIGATION = "invalid_scheme_navigation"
    IN_PROGRESS_NAVIGATION = "in_progress_navigation"
    NAVIGATION_REQUEST_BLOCKED_BY_CSP = "navigation_request_blocked_by_csp"
    MAIN_FRAME_NAVIGATION = "main_frame_navigation"
    MOJO_BINDER_POLICY = "mojo_binder_policy"
    RENDERER_PROCESS_CRASHED = "renderer_process_crashed"
    RENDERER_PROCESS_KILLED = "renderer_process_killed"
    DOWNLOAD = "download"
    TRIGGER_DESTROYED = "trigger_destroyed"
    NAVIGATION_NOT_COMMITTED = "navigation_not_committed"
    NAVIGATION_BAD_HTTP_STATUS = "navigation_bad_http_status"
    CLIENT_CERT_REQUESTED = "client_cert_requested"
    NAVIGATION_REQUEST_NETWORK_ERROR = "navigation_request_network_error"
    MAX_NUM_OF_RUNNING_PRERENDERS_EXCEEDED = "max_num_of_running_prerenders_exceeded"
    CANCEL_ALL_HOSTS_FOR_TESTING = "cancel_all_hosts_for_testing"
    DID_FAIL_LOAD = "did_fail_load"
    STOP = "stop"
    SSL_CERTIFICATE_ERROR = "ssl_certificate_error"
    LOGIN_AUTH_REQUESTED = "login_auth_requested"
    UA_CHANGE_REQUIRES_RELOAD = "ua_change_requires_reload"
    BLOCKED_BY_CLIENT = "blocked_by_client"
    AUDIO_OUTPUT_DEVICE_REQUESTED = "audio_output_device_requested"
    MIXED_CONTENT = "mixed_content"
    TRIGGER_BACKGROUNDED = "trigger_backgrounded"
    EMBEDDER_TRIGGERED_AND_CROSS_ORIGIN_REDIRECTED = "embedder_triggered_and_cross_origin_redirected"
    MEMORY_LIMIT_EXCEEDED = "memory_limit_exceeded"
    FAIL_TO_GET_MEMORY_USAGE = "fail_to_get_memory_usage"
    DATA_SAVER_ENABLED = "data_saver_enabled"
    HAS_EFFECTIVE_URL = "has_effective_url"
    ACTIVATED_BEFORE_STARTED = "activated_before_started"
    INACTIVE_PAGE_RESTRICTION = "inactive_page_restriction"
    START_FAILED = "start_failed"
    TIMEOUT_BACKGROUNDED = "timeout_backgrounded"
    CROSS_SITE_REDIRECT_IN_INITIAL_NAVIGATION = "cross_site_redirect_in_initial_navigation"
    CROSS_SITE_NAVIGATION_IN_INITIAL_NAVIGATION = "cross_site_navigation_in_initial_navigation"
    SAME_SITE_CROSS_ORIGIN_REDIRECT_NOT_OPT_IN_IN_INITIAL_NAVIGATION = (
        "same_site_cross_origin_redirect_not_opt_in_in_initial_navigation"
    )
    SAME_SITE_CROSS_ORIGIN_NAVIGATION_NOT_OPT_IN_IN_INITIAL_NAVIGATION = (
        "same_site_cross_origin_navigation_not_opt_in_in_initial_navigation"
    )
    ACTIVATION_NAVIGATION_PARAMETER_MISMATCH = "activation_navigation_parameter_mismatch"
    ACTIVATED_IN_BACKGROUND = "activated_in_background"
    EMBEDDER_HOST_DISALLOWED = "embedder_host_disallowed"
    ACTIVATION_NAVIGATION_DESTROYED_BEFORE_SUCCESS = "activation_navigation_destroyed_before_success"
    TAB_CLOSED_BY_USER_GESTURE = "tab_closed_by_user_gesture"
    TAB_CLOSED_WITHOUT_USER_GESTURE = "tab_closed_without_user_gesture"
    PRIMARY_MAIN_FRAME_RENDERER_PROCESS_CRASHED = "primary_main_frame_renderer_process_crashed"
    PRIMARY_MAIN_FRAME_RENDERER_PROCESS_KILLED = "primary_main_frame_renderer_process_killed"
    ACTIVATION_FRAME_POLICY_NOT_COMPATIBLE = "activation_frame_policy_not_compatible"
    PRELOADING_DISABLED = "preloading_disabled"
    BATTERY_SAVER_ENABLED = "battery_saver_enabled"
    ACTIVATED_DURING_MAIN_FRAME_NAVIGATION = "activated_during_main_frame_navigation"
    PRELOADING_UNSUPPORTED_BY_WEB_CONTENTS = "preloading_unsupported_by_web_contents"
    CROSS_SITE_REDIRECT_IN_MAIN_FRAME_NAVIGATION = "cross_site_redirect_in_main_frame_navigation"
    CROSS_SITE_NAVIGATION_IN_MAIN_FRAME_NAVIGATION = "cross_site_navigation_in_main_frame_navigation"
    SAME_SITE_CROSS_ORIGIN_REDIRECT_NOT_OPT_IN_IN_MAIN_FRAME_NAVIGATION = (
        "same_site_cross_origin_redirect_not_opt_in_in_main_frame_navigation"
    )
    SAME_SITE_CROSS_ORIGIN_NAVIGATION_NOT_OPT_IN_IN_MAIN_FRAME_NAVIGATION = (
        "same_site_cross_origin_navigation_not_opt_in_in_main_frame_navigation"
    )

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


class PreloadingStatus(str, enum.Enum):
    """Preloading status values, see also PreloadingTriggeringOutcome.

    This status is shared by prefetchStatusUpdated and prerenderStatusUpdated.
    """

    PENDING = "pending"
    RUNNING = "running"
    READY = "ready"
    SUCCESS = "success"
    FAILURE = "failure"
    NOT_SUPPORTED = "not_supported"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


@dataclass
@memoize_event("Preload.ruleSetUpdated")
class RuleSetUpdated:
    """Upsert.

    Currently, it is only emitted when a rule set added.
    """

    rule_set: RuleSet


@dataclass
@memoize_event("Preload.ruleSetRemoved")
class RuleSetRemoved:
    """Description is missing from the devtools protocol document."""

    id: RuleSetId


@dataclass
@memoize_event("Preload.prerenderAttemptCompleted")
class PrerenderAttemptCompleted:
    """Fired when a prerender attempt is completed."""

    key: PreloadingAttemptKey
    initiating_frame_id: page.FrameId
    prerendering_url: str
    final_status: PrerenderFinalStatus
    disallowed_api_method: typing.Optional[str]


@dataclass
@memoize_event("Preload.prefetchStatusUpdated")
class PrefetchStatusUpdated:
    """Fired when a prefetch attempt is updated."""

    key: PreloadingAttemptKey
    initiating_frame_id: page.FrameId
    prefetch_url: str
    status: PreloadingStatus


@dataclass
@memoize_event("Preload.prerenderStatusUpdated")
class PrerenderStatusUpdated:
    """Fired when a prerender attempt is updated."""

    key: PreloadingAttemptKey
    initiating_frame_id: page.FrameId
    prerendering_url: str
    status: PreloadingStatus


@dataclass
@memoize_event("Preload.preloadingAttemptSourcesUpdated")
class PreloadingAttemptSourcesUpdated:
    """Send a list of sources for all preloading attempts in a document."""

    loader_id: network.LoaderId
    preloading_attempt_sources: typing.List[PreloadingAttemptSource]


async def enable() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def disable() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...



# THIS FILE HAS BEEN AUTOMATICALLY GENERATED.
# CHANGES TO THIS FILE ARE FUTILE AS IT WILL BE OVERWRITTEN
# WHEN THE DEVTOOLS PROTOCOL CHANGES.  IF THERE ANY BUGS
# OR YOU WISH TO CHANGE HOW THE FILES ARE GENERATED PLEASE
# REFER TO: https://github.com/symonk/python-cdp OR YOUR
# OWN FORK.  REFERENCE THE `generate.py` FILE FOR CONTEXT
# AND INSTRUCTIONS.
# Chrome Devtools Protocol Domain Mapped to: `Cast`.
# Url for domain: https://chromedevtools.github.io/devtools-protocol/tot/Cast/

from __future__ import annotations

import typing
from dataclasses import dataclass

from .utils import memoize_event


@dataclass
class Sink:
    """Description is missing from the devtools protocol document."""

    # Description is missing from the devtools protocol document. # noqa
    name: str
    # Description is missing from the devtools protocol document. # noqa
    id: str
    # Text describing the current session. Present only if there is an activesession on the sink. # noqa
    session: typing.Optional[str]


@dataclass
@memoize_event("Cast.sinksUpdated")
class SinksUpdated:
    """This is fired whenever the list of available sinks changes.

    A sink is a device or a software surface that you can cast to.
    """

    sinks: typing.List[Sink]


@dataclass
@memoize_event("Cast.issueUpdated")
class IssueUpdated:
    """This is fired whenever the outstanding issue/error message changes.

    |issueMessage| is empty if there is no issue.
    """

    issue_message: str


async def enable() -> None:
    """Starts observing for sinks that can be used for tab mirroring, and if set, sinks compatible with.

    |presentationUrl| as well. When sinks are found, a.

    |sinksUpdated| event is fired. Also starts observing for issue messages. When an issue is added or removed, an
    |issueUpdated| event is fired. # noqa
    """
    ...


async def disable() -> None:
    """Stops observing for sinks and issues.

    # noqa
    """
    ...


async def set_sink_to_use() -> None:
    """Sets a sink to be used when the web page requests the browser to choose a sink via Presentation API, Remote
    Playback API, or Cast SDK.

    # noqa
    """
    ...


async def start_desktop_mirroring() -> None:
    """Starts mirroring the desktop to the sink.

    # noqa
    """
    ...


async def start_tab_mirroring() -> None:
    """Starts mirroring the tab to the sink.

    # noqa
    """
    ...


async def stop_casting() -> None:
    """Stops the active Cast session on the sink.

    # noqa
    """
    ...



# THIS FILE HAS BEEN AUTOMATICALLY GENERATED.
# CHANGES TO THIS FILE ARE FUTILE AS IT WILL BE OVERWRITTEN
# WHEN THE DEVTOOLS PROTOCOL CHANGES.  IF THERE ANY BUGS
# OR YOU WISH TO CHANGE HOW THE FILES ARE GENERATED PLEASE
# REFER TO: https://github.com/symonk/python-cdp OR YOUR
# OWN FORK.  REFERENCE THE `generate.py` FILE FOR CONTEXT
# AND INSTRUCTIONS.
# Chrome Devtools Protocol Domain Mapped to: `Inspector`.
# Url for domain: https://chromedevtools.github.io/devtools-protocol/tot/Inspector/

from __future__ import annotations

from dataclasses import dataclass

from .utils import memoize_event


@dataclass
@memoize_event("Inspector.detached")
class Detached:
    """Fired when remote debugging connection is about to be terminated.

    Contains detach reason.
    """

    reason: str


@dataclass
@memoize_event("Inspector.targetCrashed")
class TargetCrashed:
    """Fired when debugging target has crashed."""


@dataclass
@memoize_event("Inspector.targetReloadedAfterCrash")
class TargetReloadedAfterCrash:
    """Fired when debugging target has reloaded after crash."""


async def disable() -> None:
    """Disables inspector domain notifications.

    # noqa
    """
    ...


async def enable() -> None:
    """Enables inspector domain notifications.

    # noqa
    """
    ...



# THIS FILE HAS BEEN AUTOMATICALLY GENERATED.
# CHANGES TO THIS FILE ARE FUTILE AS IT WILL BE OVERWRITTEN
# WHEN THE DEVTOOLS PROTOCOL CHANGES.  IF THERE ANY BUGS
# OR YOU WISH TO CHANGE HOW THE FILES ARE GENERATED PLEASE
# REFER TO: https://github.com/symonk/python-cdp OR YOUR
# OWN FORK.  REFERENCE THE `generate.py` FILE FOR CONTEXT
# AND INSTRUCTIONS.
# Chrome Devtools Protocol Domain Mapped to: `Database`.
# Url for domain: https://chromedevtools.github.io/devtools-protocol/tot/Database/

from __future__ import annotations

from dataclasses import dataclass

from .utils import memoize_event


class DatabaseId(str):
    """Unique identifier of Database object."""

    def to_json(self) -> DatabaseId:
        return self

    @classmethod
    def from_json(cls, value: str) -> DatabaseId:
        return cls(value)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(({super().__repr__()}))"


@dataclass
class Database:
    """Database object."""

    # Database ID. # noqa
    id: DatabaseId
    # Database domain. # noqa
    domain: str
    # Database name. # noqa
    name: str
    # Database version. # noqa
    version: str


@dataclass
class Error:
    """Database error."""

    # Error message. # noqa
    message: str
    # Error code. # noqa
    code: int


@dataclass
@memoize_event("Database.addDatabase")
class AddDatabase:
    """Description is missing from the devtools protocol document."""

    database: Database


async def disable() -> None:
    """Disables database tracking, prevents database events from being sent to the client.

    # noqa
    """
    ...


async def enable() -> None:
    """Enables database tracking, database events will now be delivered to the client.

    # noqa
    """
    ...


async def execute_sql() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def get_database_table_names() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...



# THIS FILE HAS BEEN AUTOMATICALLY GENERATED.
# CHANGES TO THIS FILE ARE FUTILE AS IT WILL BE OVERWRITTEN
# WHEN THE DEVTOOLS PROTOCOL CHANGES.  IF THERE ANY BUGS
# OR YOU WISH TO CHANGE HOW THE FILES ARE GENERATED PLEASE
# REFER TO: https://github.com/symonk/python-cdp OR YOUR
# OWN FORK.  REFERENCE THE `generate.py` FILE FOR CONTEXT
# AND INSTRUCTIONS.
# Chrome Devtools Protocol Domain Mapped to: `DOMStorage`.
# Url for domain: https://chromedevtools.github.io/devtools-protocol/tot/DOMStorage/

from __future__ import annotations

import typing
from dataclasses import dataclass

from .utils import memoize_event


class SerializedStorageKey(str):
    """Description is missing from the devtools protocol document."""

    def to_json(self) -> SerializedStorageKey:
        return self

    @classmethod
    def from_json(cls, value: str) -> SerializedStorageKey:
        return cls(value)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(({super().__repr__()}))"


@dataclass
class StorageId:
    """DOM Storage identifier."""

    # Whether the storage is local storage (not session storage). # noqa
    is_local_storage: bool
    # Security origin for the storage. # noqa
    security_origin: typing.Optional[str]
    # Represents a key by which DOM Storage keys its CachedStorageAreas # noqa
    storage_key: typing.Optional[SerializedStorageKey]


@dataclass
class Item:
    """DOM Storage item."""


@dataclass
@memoize_event("DOMStorage.domStorageItemAdded")
class DomStorageItemAdded:
    """Description is missing from the devtools protocol document."""

    storage_id: StorageId
    key: str
    new_value: str


@dataclass
@memoize_event("DOMStorage.domStorageItemRemoved")
class DomStorageItemRemoved:
    """Description is missing from the devtools protocol document."""

    storage_id: StorageId
    key: str


@dataclass
@memoize_event("DOMStorage.domStorageItemUpdated")
class DomStorageItemUpdated:
    """Description is missing from the devtools protocol document."""

    storage_id: StorageId
    key: str
    old_value: str
    new_value: str


@dataclass
@memoize_event("DOMStorage.domStorageItemsCleared")
class DomStorageItemsCleared:
    """Description is missing from the devtools protocol document."""

    storage_id: StorageId


async def clear() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def disable() -> None:
    """Disables storage tracking, prevents storage events from being sent to the client.

    # noqa
    """
    ...


async def enable() -> None:
    """Enables storage tracking, storage events will now be delivered to the client.

    # noqa
    """
    ...


async def get_dom_storage_items() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def remove_dom_storage_item() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def set_dom_storage_item() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...



# THIS FILE HAS BEEN AUTOMATICALLY GENERATED.
# CHANGES TO THIS FILE ARE FUTILE AS IT WILL BE OVERWRITTEN
# WHEN THE DEVTOOLS PROTOCOL CHANGES.  IF THERE ANY BUGS
# OR YOU WISH TO CHANGE HOW THE FILES ARE GENERATED PLEASE
# REFER TO: https://github.com/symonk/python-cdp OR YOUR
# OWN FORK.  REFERENCE THE `generate.py` FILE FOR CONTEXT
# AND INSTRUCTIONS.
# Chrome Devtools Protocol Domain Mapped to: `FedCm`.
# Url for domain: https://chromedevtools.github.io/devtools-protocol/tot/FedCm/

from __future__ import annotations

import enum
import typing
from dataclasses import dataclass

from .utils import memoize_event


class LoginState(str, enum.Enum):
    """Whether this is a sign-up or sign-in action for this account, i.e. whether this account has ever been used to
    sign in to this RP before."""

    SIGN_IN = "sign_in"
    SIGN_UP = "sign_up"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


@dataclass
class Account:
    """Corresponds to IdentityRequestAccount."""

    # Description is missing from the devtools protocol document. # noqa
    account_id: str
    # Description is missing from the devtools protocol document. # noqa
    email: str
    # Description is missing from the devtools protocol document. # noqa
    name: str
    # Description is missing from the devtools protocol document. # noqa
    given_name: str
    # Description is missing from the devtools protocol document. # noqa
    picture_url: str
    # Description is missing from the devtools protocol document. # noqa
    idp_config_url: str
    # Description is missing from the devtools protocol document. # noqa
    idp_signin_url: str
    # Description is missing from the devtools protocol document. # noqa
    login_state: LoginState
    # These two are only set if the loginState is signUp # noqa
    terms_of_service_url: typing.Optional[str]
    # Description is missing from the devtools protocol document. # noqa
    privacy_policy_url: typing.Optional[str]


@dataclass
@memoize_event("FedCm.dialogShown")
class DialogShown:
    """Description is missing from the devtools protocol document."""

    dialog_id: str
    accounts: typing.List[Account]
    title: str
    subtitle: typing.Optional[str]


async def enable() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def disable() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def select_account() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def dismiss_dialog() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def reset_cooldown() -> None:
    """Resets the cooldown time, if any, to allow the next FedCM call to show a dialog even if one was recently
    dismissed by the user.

    # noqa
    """
    ...



# THIS FILE HAS BEEN AUTOMATICALLY GENERATED.
# CHANGES TO THIS FILE ARE FUTILE AS IT WILL BE OVERWRITTEN
# WHEN THE DEVTOOLS PROTOCOL CHANGES.  IF THERE ANY BUGS
# OR YOU WISH TO CHANGE HOW THE FILES ARE GENERATED PLEASE
# REFER TO: https://github.com/symonk/python-cdp OR YOUR
# OWN FORK.  REFERENCE THE `generate.py` FILE FOR CONTEXT
# AND INSTRUCTIONS.
# Chrome Devtools Protocol Domain Mapped to: `IndexedDB`.
# Url for domain: https://chromedevtools.github.io/devtools-protocol/tot/IndexedDB/

from __future__ import annotations

import typing
from dataclasses import dataclass

from . import runtime


@dataclass
class DatabaseWithObjectStores:
    """Database with an array of object stores."""

    # Database name. # noqa
    name: str
    # Database version (type is not 'integer', as the standard requires theversion number to be 'unsigned long long') # noqa
    version: float
    # Object stores in this database. # noqa
    object_stores: ObjectStore


@dataclass
class ObjectStore:
    """Object store."""

    # Object store name. # noqa
    name: str
    # Object store key path. # noqa
    key_path: KeyPath
    # If true, object store has auto increment flag set. # noqa
    auto_increment: bool
    # Indexes in this object store. # noqa
    indexes: ObjectStoreIndex


@dataclass
class ObjectStoreIndex:
    """Object store index."""

    # Index name. # noqa
    name: str
    # Index key path. # noqa
    key_path: KeyPath
    # If true, index is unique. # noqa
    unique: bool
    # If true, index allows multiple entries for a key. # noqa
    multi_entry: bool


@dataclass
class Key:
    """Key."""

    # Key type. # noqa
    type: typing.List[typing.Literal["number", "string", "date", "array"]]
    # Number value. # noqa
    number: typing.Optional[float]
    # String value. # noqa
    string: typing.Optional[str]
    # Date value. # noqa
    date: typing.Optional[float]
    # Array value. # noqa
    array: typing.Optional[Key]


@dataclass
class KeyRange:
    """Key range."""

    # If true lower bound is open. # noqa
    lower_open: bool
    # If true upper bound is open. # noqa
    upper_open: bool
    # Lower bound. # noqa
    lower: typing.Optional[Key]
    # Upper bound. # noqa
    upper: typing.Optional[Key]


@dataclass
class DataEntry:
    """Data entry."""

    # Key object. # noqa
    key: runtime.RemoteObject
    # Primary key object. # noqa
    primary_key: runtime.RemoteObject
    # Value object. # noqa
    value: runtime.RemoteObject


@dataclass
class KeyPath:
    """Key path."""

    # Key path type. # noqa
    type: typing.List[typing.Literal["null", "string", "array"]]
    # String value. # noqa
    string: typing.Optional[str]
    # Array value. # noqa
    array: typing.Optional[str]


async def clear_object_store() -> None:
    """Clears all entries from an object store.

    # noqa
    """
    ...


async def delete_database() -> None:
    """Deletes a database.

    # noqa
    """
    ...


async def delete_object_store_entries() -> None:
    """Delete a range of entries from an object store # noqa."""
    ...


async def disable() -> None:
    """Disables events from backend.

    # noqa
    """
    ...


async def enable() -> None:
    """Enables events from backend.

    # noqa
    """
    ...


async def request_data() -> None:
    """Requests data from object store or index.

    # noqa
    """
    ...


async def get_metadata() -> None:
    """Gets metadata of an object store.

    # noqa
    """
    ...


async def request_database() -> None:
    """Requests database with given name in given frame.

    # noqa
    """
    ...


async def request_database_names() -> None:
    """Requests database names for given security origin.

    # noqa
    """
    ...



# THIS FILE HAS BEEN AUTOMATICALLY GENERATED.
# CHANGES TO THIS FILE ARE FUTILE AS IT WILL BE OVERWRITTEN
# WHEN THE DEVTOOLS PROTOCOL CHANGES.  IF THERE ANY BUGS
# OR YOU WISH TO CHANGE HOW THE FILES ARE GENERATED PLEASE
# REFER TO: https://github.com/symonk/python-cdp OR YOUR
# OWN FORK.  REFERENCE THE `generate.py` FILE FOR CONTEXT
# AND INSTRUCTIONS.
# Chrome Devtools Protocol Domain Mapped to: `Browser`.
# Url for domain: https://chromedevtools.github.io/devtools-protocol/tot/Browser/

from __future__ import annotations

import enum
import typing
from dataclasses import dataclass

from . import page
from .utils import memoize_event


class BrowserContextID(str):
    """Description is missing from the devtools protocol document."""

    def to_json(self) -> BrowserContextID:
        return self

    @classmethod
    def from_json(cls, value: str) -> BrowserContextID:
        return cls(value)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(({super().__repr__()}))"


class WindowID(int):
    """Description is missing from the devtools protocol document."""

    def to_json(self) -> WindowID:
        return self

    @classmethod
    def from_json(cls, value: int) -> WindowID:
        return cls(value)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(({super().__repr__()}))"


class WindowState(str, enum.Enum):
    """The state of the browser window."""

    NORMAL = "normal"
    MINIMIZED = "minimized"
    MAXIMIZED = "maximized"
    FULLSCREEN = "fullscreen"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


@dataclass
class Bounds:
    """Browser window bounds information."""

    # The offset from the left edge of the screen to the window in pixels. # noqa
    left: typing.Optional[int]
    # The offset from the top edge of the screen to the window in pixels. # noqa
    top: typing.Optional[int]
    # The window width in pixels. # noqa
    width: typing.Optional[int]
    # The window height in pixels. # noqa
    height: typing.Optional[int]
    # The window state. Default to normal. # noqa
    window_state: typing.Optional[WindowState]


class PermissionType(str, enum.Enum):
    """Description is missing from the devtools protocol document."""

    ACCESSIBILITY_EVENTS = "accessibility_events"
    AUDIO_CAPTURE = "audio_capture"
    BACKGROUND_SYNC = "background_sync"
    BACKGROUND_FETCH = "background_fetch"
    CLIPBOARD_READ_WRITE = "clipboard_read_write"
    CLIPBOARD_SANITIZED_WRITE = "clipboard_sanitized_write"
    DISPLAY_CAPTURE = "display_capture"
    DURABLE_STORAGE = "durable_storage"
    FLASH = "flash"
    GEOLOCATION = "geolocation"
    IDLE_DETECTION = "idle_detection"
    LOCAL_FONTS = "local_fonts"
    MIDI = "midi"
    MIDI_SYSEX = "midi_sysex"
    NFC = "nfc"
    NOTIFICATIONS = "notifications"
    PAYMENT_HANDLER = "payment_handler"
    PERIODIC_BACKGROUND_SYNC = "periodic_background_sync"
    PROTECTED_MEDIA_IDENTIFIER = "protected_media_identifier"
    SENSORS = "sensors"
    STORAGE_ACCESS = "storage_access"
    TOP_LEVEL_STORAGE_ACCESS = "top_level_storage_access"
    VIDEO_CAPTURE = "video_capture"
    VIDEO_CAPTURE_PAN_TILT_ZOOM = "video_capture_pan_tilt_zoom"
    WAKE_LOCK_SCREEN = "wake_lock_screen"
    WAKE_LOCK_SYSTEM = "wake_lock_system"
    WINDOW_MANAGEMENT = "window_management"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


class PermissionSetting(str, enum.Enum):
    """Description is missing from the devtools protocol document."""

    GRANTED = "granted"
    DENIED = "denied"
    PROMPT = "prompt"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


@dataclass
class PermissionDescriptor:
    """Definition of PermissionDescriptor defined in the Permissions API:

    https://w3c.github.io/permissions/#dictdef-permissiondescriptor.
    """

    # Name of permission. See https://cs.chromium.org/chromium/src/third_party/blink/renderer/modules/permissions/permission_descriptor.idl for valid permissionnames. # noqa
    name: str
    # For "midi" permission, may also specify sysex control. # noqa
    sysex: typing.Optional[bool]
    # For "push" permission, may specify userVisibleOnly. Note thatuserVisibleOnly = true is the only currently supported type. # noqa
    user_visible_only: typing.Optional[bool]
    # For "clipboard" permission, may specify allowWithoutSanitization. # noqa
    allow_without_sanitization: typing.Optional[bool]
    # For "camera" permission, may specify panTiltZoom. # noqa
    pan_tilt_zoom: typing.Optional[bool]


class BrowserCommandId(str, enum.Enum):
    """Browser command ids used by executeBrowserCommand."""

    OPEN_TAB_SEARCH = "open_tab_search"
    CLOSE_TAB_SEARCH = "close_tab_search"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


@dataclass
class Bucket:
    """Chrome histogram bucket."""

    # Minimum value (inclusive). # noqa
    low: int
    # Maximum value (exclusive). # noqa
    high: int
    # Number of samples. # noqa
    count: int


@dataclass
class Histogram:
    """Chrome histogram."""

    # Name. # noqa
    name: str
    # Sum of sample values. # noqa
    sum: int
    # Total number of samples. # noqa
    count: int
    # Buckets. # noqa
    buckets: Bucket


@dataclass
@memoize_event("Browser.downloadWillBegin")
class DownloadWillBegin:
    """Fired when page is about to start a download."""

    frame_id: page.FrameId
    guid: str
    url: str
    suggested_filename: str


@dataclass
@memoize_event("Browser.downloadProgress")
class DownloadProgress:
    """Fired when download makes progress.

    Last call has |done| == true.
    """

    guid: str
    total_bytes: float
    received_bytes: float
    state: typing.Literal["in_progress", "completed", "canceled"]


async def set_permission() -> None:
    """Set permission settings for given origin.

    # noqa
    """
    ...


async def grant_permissions() -> None:
    """Grant specific permissions to the given origin and reject all others.

    # noqa
    """
    ...


async def reset_permissions() -> None:
    """Reset all permission management for all origins.

    # noqa
    """
    ...


async def set_download_behavior() -> None:
    """Set the behavior when downloading a file.

    # noqa
    """
    ...


async def cancel_download() -> None:
    """Cancel a download if in progress # noqa."""
    ...


async def close() -> None:
    """Close browser gracefully.

    # noqa
    """
    ...


async def crash() -> None:
    """Crashes browser on the main thread.

    # noqa
    """
    ...


async def crash_gpu_process() -> None:
    """Crashes GPU process.

    # noqa
    """
    ...


async def get_version() -> None:
    """Returns version information.

    # noqa
    """
    ...


async def get_browser_command_line() -> None:
    """Returns the command line switches for the browser process if, and only if.

    --enable-automation is on the commandline. # noqa
    """
    ...


async def get_histograms() -> None:
    """Get Chrome histograms.

    # noqa
    """
    ...


async def get_histogram() -> None:
    """Get a Chrome histogram by name.

    # noqa
    """
    ...


async def get_window_bounds() -> None:
    """Get position and size of the browser window.

    # noqa
    """
    ...


async def get_window_for_target() -> None:
    """Get the browser window that contains the devtools target.

    # noqa
    """
    ...


async def set_window_bounds() -> None:
    """Set position and/or size of the browser window.

    # noqa
    """
    ...


async def set_dock_tile() -> None:
    """Set dock tile details, platform-specific.

    # noqa
    """
    ...


async def execute_browser_command() -> None:
    """Invoke custom browser commands used by telemetry.

    # noqa
    """
    ...



# THIS FILE HAS BEEN AUTOMATICALLY GENERATED.
# CHANGES TO THIS FILE ARE FUTILE AS IT WILL BE OVERWRITTEN
# WHEN THE DEVTOOLS PROTOCOL CHANGES.  IF THERE ANY BUGS
# OR YOU WISH TO CHANGE HOW THE FILES ARE GENERATED PLEASE
# REFER TO: https://github.com/symonk/python-cdp OR YOUR
# OWN FORK.  REFERENCE THE `generate.py` FILE FOR CONTEXT
# AND INSTRUCTIONS.
# Chrome Devtools Protocol Domain Mapped to: `HeapProfiler`.
# Url for domain: https://chromedevtools.github.io/devtools-protocol/tot/HeapProfiler/

from __future__ import annotations

import typing
from dataclasses import dataclass

from . import runtime
from .utils import memoize_event


class HeapSnapshotObjectId(str):
    """Heap snapshot object id."""

    def to_json(self) -> HeapSnapshotObjectId:
        return self

    @classmethod
    def from_json(cls, value: str) -> HeapSnapshotObjectId:
        return cls(value)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(({super().__repr__()}))"


@dataclass
class SamplingHeapProfileNode:
    """Sampling Heap Profile node.

    Holds callsite information, allocation statistics and child nodes.
    """

    # Function location. # noqa
    call_frame: runtime.CallFrame
    # Allocations size in bytes for the node excluding children. # noqa
    self_size: float
    # Node id. Ids are unique across all profiles collected betweenstartSampling and stopSampling. # noqa
    id: int
    # Child nodes. # noqa
    children: SamplingHeapProfileNode


@dataclass
class SamplingHeapProfileSample:
    """A single sample from a sampling profile."""

    # Allocation size in bytes attributed to the sample. # noqa
    size: float
    # Id of the corresponding profile tree node. # noqa
    node_id: int
    # Time-ordered sample ordinal number. It is unique across all profilesretrieved between startSampling and stopSampling. # noqa
    ordinal: float


@dataclass
class SamplingHeapProfile:
    """Sampling profile."""

    # Description is missing from the devtools protocol document. # noqa
    head: SamplingHeapProfileNode
    # Description is missing from the devtools protocol document. # noqa
    samples: SamplingHeapProfileSample


@dataclass
@memoize_event("HeapProfiler.addHeapSnapshotChunk")
class AddHeapSnapshotChunk:
    """Description is missing from the devtools protocol document."""

    chunk: str


@dataclass
@memoize_event("HeapProfiler.heapStatsUpdate")
class HeapStatsUpdate:
    """If heap objects tracking has been started then backend may send update for one or more fragments."""

    stats_update: typing.List[int]


@dataclass
@memoize_event("HeapProfiler.lastSeenObjectId")
class LastSeenObjectId:
    """If heap objects tracking has been started then backend regularly sends a current value for last seen object id
    and corresponding timestamp.

    If the were changes in the heap since last event then one or more heapStatsUpdate events will be sent before a new
    lastSeenObjectId event.
    """

    last_seen_object_id: int
    timestamp: float


@dataclass
@memoize_event("HeapProfiler.reportHeapSnapshotProgress")
class ReportHeapSnapshotProgress:
    """Description is missing from the devtools protocol document."""

    done: int
    total: int
    finished: typing.Optional[bool]


@dataclass
@memoize_event("HeapProfiler.resetProfiles")
class ResetProfiles:
    """Description is missing from the devtools protocol document."""


async def add_inspected_heap_object() -> None:
    """Enables console to refer to the node with given id via $x (see Command Line API for more details.

    $x functions). # noqa
    """
    ...


async def collect_garbage() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def disable() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def enable() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def get_heap_object_id() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def get_object_by_heap_object_id() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def get_sampling_profile() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def start_sampling() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def start_tracking_heap_objects() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def stop_sampling() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def stop_tracking_heap_objects() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def take_heap_snapshot() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...



# THIS FILE HAS BEEN AUTOMATICALLY GENERATED.
# CHANGES TO THIS FILE ARE FUTILE AS IT WILL BE OVERWRITTEN
# WHEN THE DEVTOOLS PROTOCOL CHANGES.  IF THERE ANY BUGS
# OR YOU WISH TO CHANGE HOW THE FILES ARE GENERATED PLEASE
# REFER TO: https://github.com/symonk/python-cdp OR YOUR
# OWN FORK.  REFERENCE THE `generate.py` FILE FOR CONTEXT
# AND INSTRUCTIONS.
# Chrome Devtools Protocol Domain Mapped to: `PerformanceTimeline`.
# Url for domain: https://chromedevtools.github.io/devtools-protocol/tot/PerformanceTimeline/

from __future__ import annotations

import typing
from dataclasses import dataclass

from . import dom
from . import network
from . import page
from .utils import memoize_event


@dataclass
class LargestContentfulPaint:
    """See https://github.com/WICG/LargestContentfulPaint and largest_contentful_paint.idl."""

    # Description is missing from the devtools protocol document. # noqa
    render_time: network.TimeSinceEpoch
    # Description is missing from the devtools protocol document. # noqa
    load_time: network.TimeSinceEpoch
    # The number of pixels being painted. # noqa
    size: float
    # The id attribute of the element, if available. # noqa
    element_id: typing.Optional[str]
    # The URL of the image (may be trimmed). # noqa
    url: typing.Optional[str]
    # Description is missing from the devtools protocol document. # noqa
    node_id: typing.Optional[dom.BackendNodeId]


@dataclass
class LayoutShiftAttribution:
    """Description is missing from the devtools protocol document."""

    # Description is missing from the devtools protocol document. # noqa
    previous_rect: dom.Rect
    # Description is missing from the devtools protocol document. # noqa
    current_rect: dom.Rect
    # Description is missing from the devtools protocol document. # noqa
    node_id: typing.Optional[dom.BackendNodeId]


@dataclass
class LayoutShift:
    """See https://wicg.github.io/layout-instability/#sec-layout-shift and layout_shift.idl."""

    # Score increment produced by this event. # noqa
    value: float
    # Description is missing from the devtools protocol document. # noqa
    had_recent_input: bool
    # Description is missing from the devtools protocol document. # noqa
    last_input_time: network.TimeSinceEpoch
    # Description is missing from the devtools protocol document. # noqa
    sources: LayoutShiftAttribution


@dataclass
class TimelineEvent:
    """Description is missing from the devtools protocol document."""

    # Identifies the frame that this event is related to. Empty for non-frametargets. # noqa
    frame_id: page.FrameId
    # The event type, as specified in https://w3c.github.io/performance-timeline/#dom-performanceentry-entrytype This determines which of the optional"details" fiedls is present. # noqa
    type: str
    # Name may be empty depending on the type. # noqa
    name: str
    # Time in seconds since Epoch, monotonically increasing within documentlifetime. # noqa
    time: network.TimeSinceEpoch
    # Event duration, if applicable. # noqa
    duration: typing.Optional[float]
    # Description is missing from the devtools protocol document. # noqa
    lcp_details: typing.Optional[LargestContentfulPaint]
    # Description is missing from the devtools protocol document. # noqa
    layout_shift_details: typing.Optional[LayoutShift]


@dataclass
@memoize_event("PerformanceTimeline.timelineEventAdded")
class TimelineEventAdded:
    """Sent when a performance timeline event is added.

    See reportPerformanceTimeline method.
    """

    event: TimelineEvent


async def enable() -> None:
    """Previously buffered events would be reported before method returns.

    See also: timelineEventAdded # noqa
    """
    ...



# THIS FILE HAS BEEN AUTOMATICALLY GENERATED.
# CHANGES TO THIS FILE ARE FUTILE AS IT WILL BE OVERWRITTEN
# WHEN THE DEVTOOLS PROTOCOL CHANGES.  IF THERE ANY BUGS
# OR YOU WISH TO CHANGE HOW THE FILES ARE GENERATED PLEASE
# REFER TO: https://github.com/symonk/python-cdp OR YOUR
# OWN FORK.  REFERENCE THE `generate.py` FILE FOR CONTEXT
# AND INSTRUCTIONS.
# Chrome Devtools Protocol Domain Mapped to: `ServiceWorker`.
# Url for domain: https://chromedevtools.github.io/devtools-protocol/tot/ServiceWorker/

from __future__ import annotations

import enum
import typing
from dataclasses import dataclass

from . import target
from .utils import memoize_event


class RegistrationID(str):
    """Description is missing from the devtools protocol document."""

    def to_json(self) -> RegistrationID:
        return self

    @classmethod
    def from_json(cls, value: str) -> RegistrationID:
        return cls(value)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(({super().__repr__()}))"


@dataclass
class ServiceWorkerRegistration:
    """ServiceWorker registration."""

    # Description is missing from the devtools protocol document. # noqa
    registration_id: RegistrationID
    # Description is missing from the devtools protocol document. # noqa
    scope_url: str
    # Description is missing from the devtools protocol document. # noqa
    is_deleted: bool


class ServiceWorkerVersionRunningStatus(str, enum.Enum):
    """Description is missing from the devtools protocol document."""

    STOPPED = "stopped"
    STARTING = "starting"
    RUNNING = "running"
    STOPPING = "stopping"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


class ServiceWorkerVersionStatus(str, enum.Enum):
    """Description is missing from the devtools protocol document."""

    NEW = "new"
    INSTALLING = "installing"
    INSTALLED = "installed"
    ACTIVATING = "activating"
    ACTIVATED = "activated"
    REDUNDANT = "redundant"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


@dataclass
class ServiceWorkerVersion:
    """ServiceWorker version."""

    # Description is missing from the devtools protocol document. # noqa
    version_id: str
    # Description is missing from the devtools protocol document. # noqa
    registration_id: RegistrationID
    # Description is missing from the devtools protocol document. # noqa
    script_url: str
    # Description is missing from the devtools protocol document. # noqa
    running_status: ServiceWorkerVersionRunningStatus
    # Description is missing from the devtools protocol document. # noqa
    status: ServiceWorkerVersionStatus
    # The Last-Modified header value of the main script. # noqa
    script_last_modified: typing.Optional[float]
    # The time at which the response headers of the main script were receivedfrom the server. For cached script it is the last time the cache entry wasvalidated. # noqa
    script_response_time: typing.Optional[float]
    # Description is missing from the devtools protocol document. # noqa
    controlled_clients: typing.Optional[target.TargetID]
    # Description is missing from the devtools protocol document. # noqa
    target_id: typing.Optional[target.TargetID]


@dataclass
class ServiceWorkerErrorMessage:
    """ServiceWorker error message."""

    # Description is missing from the devtools protocol document. # noqa
    error_message: str
    # Description is missing from the devtools protocol document. # noqa
    registration_id: RegistrationID
    # Description is missing from the devtools protocol document. # noqa
    version_id: str
    # Description is missing from the devtools protocol document. # noqa
    source_url: str
    # Description is missing from the devtools protocol document. # noqa
    line_number: int
    # Description is missing from the devtools protocol document. # noqa
    column_number: int


@dataclass
@memoize_event("ServiceWorker.workerErrorReported")
class WorkerErrorReported:
    """Description is missing from the devtools protocol document."""

    error_message: ServiceWorkerErrorMessage


@dataclass
@memoize_event("ServiceWorker.workerRegistrationUpdated")
class WorkerRegistrationUpdated:
    """Description is missing from the devtools protocol document."""

    registrations: typing.List[ServiceWorkerRegistration]


@dataclass
@memoize_event("ServiceWorker.workerVersionUpdated")
class WorkerVersionUpdated:
    """Description is missing from the devtools protocol document."""

    versions: typing.List[ServiceWorkerVersion]


async def deliver_push_message() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def disable() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def dispatch_sync_event() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def dispatch_periodic_sync_event() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def enable() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def inspect_worker() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def set_force_update_on_page_load() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def skip_waiting() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def start_worker() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def stop_all_workers() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def stop_worker() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def unregister() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def update_registration() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...



# THIS FILE HAS BEEN AUTOMATICALLY GENERATED.
# CHANGES TO THIS FILE ARE FUTILE AS IT WILL BE OVERWRITTEN
# WHEN THE DEVTOOLS PROTOCOL CHANGES.  IF THERE ANY BUGS
# OR YOU WISH TO CHANGE HOW THE FILES ARE GENERATED PLEASE
# REFER TO: https://github.com/symonk/python-cdp OR YOUR
# OWN FORK.  REFERENCE THE `generate.py` FILE FOR CONTEXT
# AND INSTRUCTIONS.
# Chrome Devtools Protocol Domain Mapped to: `Animation`.
# Url for domain: https://chromedevtools.github.io/devtools-protocol/tot/Animation/

from __future__ import annotations

import typing
from dataclasses import dataclass

from . import dom
from .utils import memoize_event


@dataclass
class Animation:
    """Animation instance."""

    # `Animation`'s id. # noqa
    id: str
    # `Animation`'s name. # noqa
    name: str
    # `Animation`'s internal paused state. # noqa
    paused_state: bool
    # `Animation`'s play state. # noqa
    play_state: str
    # `Animation`'s playback rate. # noqa
    playback_rate: float
    # `Animation`'s start time. # noqa
    start_time: float
    # `Animation`'s current time. # noqa
    current_time: float
    # Animation type of `Animation`. # noqa
    type: typing.List[typing.Literal["CSSTransition", "CSSAnimation", "WebAnimation"]]
    # `Animation`'s source animation node. # noqa
    source: typing.Optional[AnimationEffect]
    # A unique ID for `Animation` representing the sources that triggered thisCSS animation/transition. # noqa
    css_id: typing.Optional[str]


@dataclass
class AnimationEffect:
    """AnimationEffect instance."""

    # `AnimationEffect`'s delay. # noqa
    delay: float
    # `AnimationEffect`'s end delay. # noqa
    end_delay: float
    # `AnimationEffect`'s iteration start. # noqa
    iteration_start: float
    # `AnimationEffect`'s iterations. # noqa
    iterations: float
    # `AnimationEffect`'s iteration duration. # noqa
    duration: float
    # `AnimationEffect`'s playback direction. # noqa
    direction: str
    # `AnimationEffect`'s fill mode. # noqa
    fill: str
    # `AnimationEffect`'s timing function. # noqa
    easing: str
    # `AnimationEffect`'s target node. # noqa
    backend_node_id: typing.Optional[dom.BackendNodeId]
    # `AnimationEffect`'s keyframes. # noqa
    keyframes_rule: typing.Optional[KeyframesRule]


@dataclass
class KeyframesRule:
    """Keyframes Rule."""

    # List of animation keyframes. # noqa
    keyframes: KeyframeStyle
    # CSS keyframed animation's name. # noqa
    name: typing.Optional[str]


@dataclass
class KeyframeStyle:
    """Keyframe Style."""

    # Keyframe's time offset. # noqa
    offset: str
    # `AnimationEffect`'s timing function. # noqa
    easing: str


@dataclass
@memoize_event("Animation.animationCanceled")
class AnimationCanceled:
    """Event for when an animation has been cancelled."""

    id: str


@dataclass
@memoize_event("Animation.animationCreated")
class AnimationCreated:
    """Event for each animation that has been created."""

    id: str


@dataclass
@memoize_event("Animation.animationStarted")
class AnimationStarted:
    """Event for animation that has been started."""

    animation: Animation


async def disable() -> None:
    """Disables animation domain notifications.

    # noqa
    """
    ...


async def enable() -> None:
    """Enables animation domain notifications.

    # noqa
    """
    ...


async def get_current_time() -> None:
    """Returns the current time of the an animation.

    # noqa
    """
    ...


async def get_playback_rate() -> None:
    """Gets the playback rate of the document timeline.

    # noqa
    """
    ...


async def release_animations() -> None:
    """Releases a set of animations to no longer be manipulated.

    # noqa
    """
    ...


async def resolve_animation() -> None:
    """Gets the remote object of the Animation.

    # noqa
    """
    ...


async def seek_animations() -> None:
    """Seek a set of animations to a particular time within each animation.

    # noqa
    """
    ...


async def set_paused() -> None:
    """Sets the paused state of a set of animations.

    # noqa
    """
    ...


async def set_playback_rate() -> None:
    """Sets the playback rate of the document timeline.

    # noqa
    """
    ...


async def set_timing() -> None:
    """Sets the timing of an animation node.

    # noqa
    """
    ...



"""This module is just a place holder for now to experiment with websockets etc."""

import asyncio
import json
from pprint import pprint as print

import websockets


class CdpClient:
    """An asynchronous client that can dispatch commands to a remote debuggable instance of any CDP enabled browser."""


async def hello():
    # This is a dummy implementation for messing around with the CDP.
    async with websockets.connect(
        "ws://127.0.0.1:9222/devtools/browser/d456053b-d450-4b42-89dd-d8d5ed5a5898",
    ) as socket:
        await socket.send(json.dumps({"id": 1, "method": "Target.getTargets", "params": {}}))
        targets = json.loads(await socket.recv())
        page_target = [t for t in targets["result"]["targetInfos"] if t["type"] == "page"][0]["targetId"]
        await socket.send(
            json.dumps(
                {"id": 2, "method": "Target.attachToTarget", "params": {"targetId": page_target, "flatten": True}},
            ),
        )
        session_id = json.loads(await socket.recv())["params"]["sessionId"]
        await socket.send(json.dumps({"sessionId": session_id, "id": 3, "method": "Network.enable", "params": {}}))
        while True:
            print(json.loads(await socket.recv()))


asyncio.run(hello())



# THIS FILE HAS BEEN AUTOMATICALLY GENERATED.
# CHANGES TO THIS FILE ARE FUTILE AS IT WILL BE OVERWRITTEN
# WHEN THE DEVTOOLS PROTOCOL CHANGES.  IF THERE ANY BUGS
# OR YOU WISH TO CHANGE HOW THE FILES ARE GENERATED PLEASE
# REFER TO: https://github.com/symonk/python-cdp OR YOUR
# OWN FORK.  REFERENCE THE `generate.py` FILE FOR CONTEXT
# AND INSTRUCTIONS.
# Chrome Devtools Protocol Domain Mapped to: `Target`.
# Url for domain: https://chromedevtools.github.io/devtools-protocol/tot/Target/

from __future__ import annotations

import typing
from dataclasses import dataclass

from . import browser
from . import page
from .utils import memoize_event


class TargetID(str):
    """Description is missing from the devtools protocol document."""

    def to_json(self) -> TargetID:
        return self

    @classmethod
    def from_json(cls, value: str) -> TargetID:
        return cls(value)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(({super().__repr__()}))"


class SessionID(str):
    """Unique identifier of attached debugging session."""

    def to_json(self) -> SessionID:
        return self

    @classmethod
    def from_json(cls, value: str) -> SessionID:
        return cls(value)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(({super().__repr__()}))"


@dataclass
class TargetInfo:
    """Description is missing from the devtools protocol document."""

    # Description is missing from the devtools protocol document. # noqa
    target_id: TargetID
    # Description is missing from the devtools protocol document. # noqa
    type: str
    # Description is missing from the devtools protocol document. # noqa
    title: str
    # Description is missing from the devtools protocol document. # noqa
    url: str
    # Whether the target has an attached client. # noqa
    attached: bool
    # Whether the target has access to the originating window. # noqa
    can_access_opener: bool
    # Opener target Id # noqa
    opener_id: typing.Optional[TargetID]
    # Frame id of originating window (is only set if target has an opener). # noqa
    opener_frame_id: typing.Optional[page.FrameId]
    # Description is missing from the devtools protocol document. # noqa
    browser_context_id: typing.Optional[browser.BrowserContextID]
    # Provides additional details for specific target types. For example, forthe type of "page", this may be set to "portal" or "prerender". # noqa
    subtype: typing.Optional[str]


@dataclass
class FilterEntry:
    """A filter used by target query/discovery/auto-attach operations."""

    # If set, causes exclusion of mathcing targets from the list. # noqa
    exclude: typing.Optional[bool]
    # If not present, matches any type. # noqa
    type: typing.Optional[str]


@dataclass
class TargetFilter:
    """The entries in TargetFilter are matched sequentially against targets and the first entry that matches determines
    if the target is included or not, depending on the value of `exclude` field in the entry. If filter is not
    specified, the one assumed is.

    [{type: "browser", exclude: true}, {type: "tab", exclude: true}, {}]
    (i.e. include everything but `browser` and `tab`).
    """


@dataclass
class RemoteLocation:
    """Description is missing from the devtools protocol document."""

    # Description is missing from the devtools protocol document. # noqa
    host: str
    # Description is missing from the devtools protocol document. # noqa
    port: int


@dataclass
@memoize_event("Target.attachedToTarget")
class AttachedToTarget:
    """Issued when attached to target because of auto-attach or `attachToTarget` command."""

    session_id: SessionID
    target_info: TargetInfo
    waiting_for_debugger: bool


@dataclass
@memoize_event("Target.detachedFromTarget")
class DetachedFromTarget:
    """Issued when detached from target for any reason (including `detachFromTarget` command).

    Can be issued multiple times per target if multiple sessions have been attached to it.
    """

    session_id: SessionID
    target_id: typing.Optional[TargetID]


@dataclass
@memoize_event("Target.receivedMessageFromTarget")
class ReceivedMessageFromTarget:
    """Notifies about a new protocol message received from the session (as reported in `attachedToTarget` event)."""

    session_id: SessionID
    message: str
    target_id: typing.Optional[TargetID]


@dataclass
@memoize_event("Target.targetCreated")
class TargetCreated:
    """Issued when a possible inspection target is created."""

    target_info: TargetInfo


@dataclass
@memoize_event("Target.targetDestroyed")
class TargetDestroyed:
    """Issued when a target is destroyed."""

    target_id: TargetID


@dataclass
@memoize_event("Target.targetCrashed")
class TargetCrashed:
    """Issued when a target has crashed."""

    target_id: TargetID
    status: str
    error_code: int


@dataclass
@memoize_event("Target.targetInfoChanged")
class TargetInfoChanged:
    """Issued when some information about a target has changed.

    This only happens between `targetCreated` and `targetDestroyed`.
    """

    target_info: TargetInfo


async def activate_target() -> None:
    """Activates (focuses) the target.

    # noqa
    """
    ...


async def attach_to_target() -> None:
    """Attaches to the target with given id.

    # noqa
    """
    ...


async def attach_to_browser_target() -> None:
    """Attaches to the browser target, only uses flat sessionId mode.

    # noqa
    """
    ...


async def close_target() -> None:
    """Closes the target.

    If the target is a page that gets closed too. # noqa
    """
    ...


async def expose_dev_tools_protocol() -> None:
    """Inject object to the target's main frame that provides a communication channel with browser target.

    Injected object will be available as `window[bindingName]`.

    The object has the follwing API:
    - `binding.send(json)` - a method to send messages over the remote debugging protocol
    - `binding.onmessage = json => handleMessage(json)` - a callback that will be called for the protocol notifications and command responses. # noqa
    """
    ...


async def create_browser_context() -> None:
    """Creates a new empty BrowserContext.

    Similar to an incognito profile but you can have more than one. # noqa
    """
    ...


async def get_browser_contexts() -> None:
    """Returns all browser contexts created with `Target.createBrowserContext` method.

    # noqa
    """
    ...


async def create_target() -> None:
    """Creates a new page.

    # noqa
    """
    ...


async def detach_from_target() -> None:
    """Detaches session with given id.

    # noqa
    """
    ...


async def dispose_browser_context() -> None:
    """Deletes a BrowserContext.

    All the belonging pages will be closed without calling their beforeunload hooks. # noqa
    """
    ...


async def get_target_info() -> None:
    """Returns information about a target.

    # noqa
    """
    ...


async def get_targets() -> None:
    """Retrieves a list of available targets.

    # noqa
    """
    ...


async def send_message_to_target() -> None:
    """Sends protocol message over session with given id.

    Consider using flat mode instead; see commands attachToTarget, setAutoAttach, and crbug.com/991325. # noqa
    """
    ...


async def set_auto_attach() -> None:
    """Controls whether to automatically attach to new targets which are considered to be related to this one.

    When turned on, attaches to all existing related targets as well. When turned off, automatically detaches from all
    currently attached targets. This also clears all targets added by `autoAttachRelated` from the list of targets to
    watch for creation of related targets. # noqa
    """
    ...


async def auto_attach_related() -> None:
    """Adds the specified target to the list of targets that will be monitored for any related target creation (such as
    child frames, child workers and new versions of service worker) and reported through `attachedToTarget`.

    The specified target is also auto-attached. This cancels the effect of any previous `setAutoAttach` and is also
    cancelled by subsequent `setAutoAttach`. Only available at the Browser target. # noqa
    """
    ...


async def set_discover_targets() -> None:
    """Controls whether to discover available targets and notify via `targetCreated/targetInfoChanged/targetDestroyed`
    events.

    # noqa
    """
    ...


async def set_remote_locations() -> None:
    """Enables target discovery for the specified locations, when `setDiscoverTargets` was set to `true`.

    # noqa
    """
    ...



# THIS FILE HAS BEEN AUTOMATICALLY GENERATED.
# CHANGES TO THIS FILE ARE FUTILE AS IT WILL BE OVERWRITTEN
# WHEN THE DEVTOOLS PROTOCOL CHANGES.  IF THERE ANY BUGS
# OR YOU WISH TO CHANGE HOW THE FILES ARE GENERATED PLEASE
# REFER TO: https://github.com/symonk/python-cdp OR YOUR
# OWN FORK.  REFERENCE THE `generate.py` FILE FOR CONTEXT
# AND INSTRUCTIONS.
# Chrome Devtools Protocol Domain Mapped to: `Runtime`.
# Url for domain: https://chromedevtools.github.io/devtools-protocol/tot/Runtime/

from __future__ import annotations

import typing
from dataclasses import dataclass

from .utils import memoize_event


class ScriptId(str):
    """Unique script identifier."""

    def to_json(self) -> ScriptId:
        return self

    @classmethod
    def from_json(cls, value: str) -> ScriptId:
        return cls(value)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(({super().__repr__()}))"


@dataclass
class WebDriverValue:
    """Represents the value serialiazed by the WebDriver BiDi specification https://w3c.github.io/webdriver-bidi."""

    # Description is missing from the devtools protocol document. # noqa
    type: typing.List[
        typing.Literal[
            "undefined",
            "null",
            "string",
            "number",
            "boolean",
            "bigint",
            "regexp",
            "date",
            "symbol",
            "array",
            "object",
            "function",
            "map",
            "set",
            "weakmap",
            "weakset",
            "error",
            "proxy",
            "promise",
            "typedarray",
            "arraybuffer",
            "node",
            "window",
        ]
    ]
    # Description is missing from the devtools protocol document. # noqa
    value: typing.Any
    # Description is missing from the devtools protocol document. # noqa
    object_id: typing.Optional[str]


class RemoteObjectId(str):
    """Unique object identifier."""

    def to_json(self) -> RemoteObjectId:
        return self

    @classmethod
    def from_json(cls, value: str) -> RemoteObjectId:
        return cls(value)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(({super().__repr__()}))"


class UnserializableValue(str):
    """Primitive value which cannot be JSON-stringified.

    Includes values `-0`, `NaN`, `Infinity`, `-Infinity`, and bigint literals.
    """

    def to_json(self) -> UnserializableValue:
        return self

    @classmethod
    def from_json(cls, value: str) -> UnserializableValue:
        return cls(value)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(({super().__repr__()}))"


@dataclass
class RemoteObject:
    """Mirror object referencing original JavaScript object."""

    # Object type. # noqa
    type: typing.List[
        typing.Literal["object", "function", "undefined", "string", "number", "boolean", "symbol", "bigint"]
    ]
    # Object subtype hint. Specified for `object` type values only. NOTE: If youchange anything here, make sure to also update `subtype` in `ObjectPreview` and`PropertyPreview` below. # noqa
    subtype: typing.Optional[
        typing.List[
            typing.Literal[
                "array",
                "null",
                "node",
                "regexp",
                "date",
                "map",
                "set",
                "weakmap",
                "weakset",
                "iterator",
                "generator",
                "error",
                "proxy",
                "promise",
                "typedarray",
                "arraybuffer",
                "dataview",
                "webassemblymemory",
                "wasmvalue",
            ]
        ]
    ]
    # Object class (constructor) name. Specified for `object` type values only. # noqa
    class_name: typing.Optional[str]
    # Remote object value in case of primitive values or JSON values (if it wasrequested). # noqa
    value: typing.Any
    # Primitive value which can not be JSON-stringified does not have `value`,but gets this property. # noqa
    unserializable_value: typing.Optional[UnserializableValue]
    # String representation of the object. # noqa
    description: typing.Optional[str]
    # WebDriver BiDi representation of the value. # noqa
    web_driver_value: typing.Optional[WebDriverValue]
    # Unique object identifier (for non-primitive values). # noqa
    object_id: typing.Optional[RemoteObjectId]
    # Preview containing abbreviated property values. Specified for `object`type values only. # noqa
    preview: typing.Optional[ObjectPreview]
    # Description is missing from the devtools protocol document. # noqa
    custom_preview: typing.Optional[CustomPreview]


@dataclass
class CustomPreview:
    """Description is missing from the devtools protocol document."""

    # The JSON-stringified result of formatter.header(object, config) call. Itcontains json ML array that represents RemoteObject. # noqa
    header: str
    # If formatter returns true as a result of formatter.hasBody call thenbodyGetterId will contain RemoteObjectId for the function that returns result offormatter.body(object, config) call. The result value is json ML array. # noqa
    body_getter_id: typing.Optional[RemoteObjectId]


@dataclass
class ObjectPreview:
    """Object containing abbreviated remote object value."""

    # Object type. # noqa
    type: typing.List[
        typing.Literal["object", "function", "undefined", "string", "number", "boolean", "symbol", "bigint"]
    ]
    # True iff some of the properties or entries of the original object did notfit. # noqa
    overflow: bool
    # List of the properties. # noqa
    properties: PropertyPreview
    # Object subtype hint. Specified for `object` type values only. # noqa
    subtype: typing.Optional[
        typing.List[
            typing.Literal[
                "array",
                "null",
                "node",
                "regexp",
                "date",
                "map",
                "set",
                "weakmap",
                "weakset",
                "iterator",
                "generator",
                "error",
                "proxy",
                "promise",
                "typedarray",
                "arraybuffer",
                "dataview",
                "webassemblymemory",
                "wasmvalue",
            ]
        ]
    ]
    # String representation of the object. # noqa
    description: typing.Optional[str]
    # List of the entries. Specified for `map` and `set` subtype values only. # noqa
    entries: typing.Optional[EntryPreview]


@dataclass
class PropertyPreview:
    """Description is missing from the devtools protocol document."""

    # Property name. # noqa
    name: str
    # Object type. Accessor means that the property itself is an accessorproperty. # noqa
    type: typing.List[
        typing.Literal["object", "function", "undefined", "string", "number", "boolean", "symbol", "accessor", "bigint"]
    ]
    # User-friendly property value string. # noqa
    value: typing.Optional[str]
    # Nested value preview. # noqa
    value_preview: typing.Optional[ObjectPreview]
    # Object subtype hint. Specified for `object` type values only. # noqa
    subtype: typing.Optional[
        typing.List[
            typing.Literal[
                "array",
                "null",
                "node",
                "regexp",
                "date",
                "map",
                "set",
                "weakmap",
                "weakset",
                "iterator",
                "generator",
                "error",
                "proxy",
                "promise",
                "typedarray",
                "arraybuffer",
                "dataview",
                "webassemblymemory",
                "wasmvalue",
            ]
        ]
    ]


@dataclass
class EntryPreview:
    """Description is missing from the devtools protocol document."""

    # Preview of the value. # noqa
    value: ObjectPreview
    # Preview of the key. Specified for map-like collection entries. # noqa
    key: typing.Optional[ObjectPreview]


@dataclass
class PropertyDescriptor:
    """Object property descriptor."""

    # Property name or symbol description. # noqa
    name: str
    # True if the type of this property descriptor may be changed and if theproperty may be deleted from the corresponding object. # noqa
    configurable: bool
    # True if this property shows up during enumeration of the properties on thecorresponding object. # noqa
    enumerable: bool
    # The value associated with the property. # noqa
    value: typing.Optional[RemoteObject]
    # True if the value associated with the property may be changed (datadescriptors only). # noqa
    writable: typing.Optional[bool]
    # A function which serves as a getter for the property, or `undefined` ifthere is no getter (accessor descriptors only). # noqa
    get: typing.Optional[RemoteObject]
    # A function which serves as a setter for the property, or `undefined` ifthere is no setter (accessor descriptors only). # noqa
    set: typing.Optional[RemoteObject]
    # True if the result was thrown during the evaluation. # noqa
    was_thrown: typing.Optional[bool]
    # True if the property is owned for the object. # noqa
    is_own: typing.Optional[bool]
    # Property symbol object, if the property is of the `symbol` type. # noqa
    symbol: typing.Optional[RemoteObject]


@dataclass
class InternalPropertyDescriptor:
    """Object internal property descriptor.

    This property isn't normally visible in JavaScript code.
    """

    # Conventional property name. # noqa
    name: str
    # The value associated with the property. # noqa
    value: typing.Optional[RemoteObject]


@dataclass
class PrivatePropertyDescriptor:
    """Object private field descriptor."""

    # Private property name. # noqa
    name: str
    # The value associated with the private property. # noqa
    value: typing.Optional[RemoteObject]
    # A function which serves as a getter for the private property, or`undefined` if there is no getter (accessor descriptors only). # noqa
    get: typing.Optional[RemoteObject]
    # A function which serves as a setter for the private property, or`undefined` if there is no setter (accessor descriptors only). # noqa
    set: typing.Optional[RemoteObject]


@dataclass
class CallArgument:
    """Represents function call argument.

    Either remote object id `objectId`, primitive `value`, unserializable primitive value or neither of (for undefined)
    them should be specified.
    """

    # Primitive value or serializable javascript object. # noqa
    value: typing.Any
    # Primitive value which can not be JSON-stringified. # noqa
    unserializable_value: typing.Optional[UnserializableValue]
    # Remote object handle. # noqa
    object_id: typing.Optional[RemoteObjectId]


class ExecutionContextId(int):
    """Id of an execution context."""

    def to_json(self) -> ExecutionContextId:
        return self

    @classmethod
    def from_json(cls, value: int) -> ExecutionContextId:
        return cls(value)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(({super().__repr__()}))"


@dataclass
class ExecutionContextDescription:
    """Description of an isolated world."""

    # Unique id of the execution context. It can be used to specify in whichexecution context script evaluation should be performed. # noqa
    id: ExecutionContextId
    # Execution context origin. # noqa
    origin: str
    # Human readable name describing given context. # noqa
    name: str
    # A system-unique execution context identifier. Unlike the id, this isunique across multiple processes, so can be reliably used to identify specificcontext while backend performs a cross-process navigation. # noqa
    unique_id: str
    # Embedder-specific auxiliary data. # noqa
    aux_data: typing.Any


@dataclass
class ExceptionDetails:
    """Detailed information about exception (or error) that was thrown during script compilation or execution."""

    # Exception id. # noqa
    exception_id: int
    # Exception text, which should be used together with exception object whenavailable. # noqa
    text: str
    # Line number of the exception location (0-based). # noqa
    line_number: int
    # Column number of the exception location (0-based). # noqa
    column_number: int
    # Script ID of the exception location. # noqa
    script_id: typing.Optional[ScriptId]
    # URL of the exception location, to be used when the script was notreported. # noqa
    url: typing.Optional[str]
    # JavaScript stack trace if available. # noqa
    stack_trace: typing.Optional[StackTrace]
    # Exception object if available. # noqa
    exception: typing.Optional[RemoteObject]
    # Identifier of the context where exception happened. # noqa
    execution_context_id: typing.Optional[ExecutionContextId]
    # Dictionary with entries of meta data that the client associated with thisexception, such as information about associated network requests, etc. # noqa
    exception_meta_data: typing.Any


class Timestamp(float):
    """Number of milliseconds since epoch."""

    def to_json(self) -> Timestamp:
        return self

    @classmethod
    def from_json(cls, value: float) -> Timestamp:
        return cls(value)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(({super().__repr__()}))"


class TimeDelta(float):
    """Number of milliseconds."""

    def to_json(self) -> TimeDelta:
        return self

    @classmethod
    def from_json(cls, value: float) -> TimeDelta:
        return cls(value)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(({super().__repr__()}))"


@dataclass
class CallFrame:
    """Stack entry for runtime errors and assertions."""

    # JavaScript function name. # noqa
    function_name: str
    # JavaScript script id. # noqa
    script_id: ScriptId
    # JavaScript script name or url. # noqa
    url: str
    # JavaScript script line number (0-based). # noqa
    line_number: int
    # JavaScript script column number (0-based). # noqa
    column_number: int


@dataclass
class StackTrace:
    """Call frames for assertions or error messages."""

    # JavaScript function name. # noqa
    call_frames: CallFrame
    # String label of this stack trace. For async traces this may be a name ofthe function that initiated the async call. # noqa
    description: typing.Optional[str]
    # Asynchronous JavaScript stack trace that preceded this stack, ifavailable. # noqa
    parent: typing.Optional[StackTrace]
    # Asynchronous JavaScript stack trace that preceded this stack, ifavailable. # noqa
    parent_id: typing.Optional[StackTraceId]


class UniqueDebuggerId(str):
    """Unique identifier of current debugger."""

    def to_json(self) -> UniqueDebuggerId:
        return self

    @classmethod
    def from_json(cls, value: str) -> UniqueDebuggerId:
        return cls(value)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(({super().__repr__()}))"


@dataclass
class StackTraceId:
    """If `debuggerId` is set stack trace comes from another debugger and can be resolved there.

    This allows to track cross-debugger calls. See `Runtime.StackTrace` and `Debugger.paused` for usages.
    """

    # Description is missing from the devtools protocol document. # noqa
    id: str
    # Description is missing from the devtools protocol document. # noqa
    debugger_id: typing.Optional[UniqueDebuggerId]


@dataclass
@memoize_event("Runtime.bindingCalled")
class BindingCalled:
    """Notification is issued every time when binding is called."""

    name: str
    payload: str
    execution_context_id: ExecutionContextId


@dataclass
@memoize_event("Runtime.consoleAPICalled")
class ConsoleAPICalled:
    """Issued when console API was called."""

    type: typing.Literal[
        "log",
        "debug",
        "info",
        "error",
        "warning",
        "dir",
        "dirxml",
        "table",
        "trace",
        "clear",
        "start_group",
        "start_group_collapsed",
        "end_group",
        "assert",
        "profile",
        "profile_end",
        "count",
        "time_end",
    ]
    args: typing.List[RemoteObject]
    execution_context_id: ExecutionContextId
    timestamp: Timestamp
    stack_trace: typing.Optional[StackTrace]
    context: typing.Optional[str]


@dataclass
@memoize_event("Runtime.exceptionRevoked")
class ExceptionRevoked:
    """Issued when unhandled exception was revoked."""

    reason: str
    exception_id: int


@dataclass
@memoize_event("Runtime.exceptionThrown")
class ExceptionThrown:
    """Issued when exception was thrown and unhandled."""

    timestamp: Timestamp
    exception_details: ExceptionDetails


@dataclass
@memoize_event("Runtime.executionContextCreated")
class ExecutionContextCreated:
    """Issued when new execution context is created."""

    context: ExecutionContextDescription


@dataclass
@memoize_event("Runtime.executionContextDestroyed")
class ExecutionContextDestroyed:
    """Issued when execution context is destroyed."""

    execution_context_id: ExecutionContextId
    execution_context_unique_id: str


@dataclass
@memoize_event("Runtime.executionContextsCleared")
class ExecutionContextsCleared:
    """Issued when all executionContexts were cleared in browser."""


@dataclass
@memoize_event("Runtime.inspectRequested")
class InspectRequested:
    """Issued when object should be inspected (for example, as a result of inspect() command line API call)."""

    object: RemoteObject
    hints: typing.Any
    execution_context_id: typing.Optional[ExecutionContextId]


async def await_promise() -> None:
    """Add handler to promise with given promise object id.

    # noqa
    """
    ...


async def call_function_on() -> None:
    """Calls function with given declaration on the given object.

    Object group of the result is inherited from the target object. # noqa
    """
    ...


async def compile_script() -> None:
    """Compiles expression.

    # noqa
    """
    ...


async def disable() -> None:
    """Disables reporting of execution contexts creation.

    # noqa
    """
    ...


async def discard_console_entries() -> None:
    """Discards collected exceptions and console API calls.

    # noqa
    """
    ...


async def enable() -> None:
    """Enables reporting of execution contexts creation by means of `executionContextCreated` event.

    When the reporting gets enabled the event will be sent immediately for each existing execution context. # noqa
    """
    ...


async def evaluate() -> None:
    """Evaluates expression on global object.

    # noqa
    """
    ...


async def get_isolate_id() -> None:
    """Returns the isolate id.

    # noqa
    """
    ...


async def get_heap_usage() -> None:
    """Returns the JavaScript heap usage.

    It is the total usage of the corresponding isolate not scoped to a particular Runtime. # noqa
    """
    ...


async def get_properties() -> None:
    """Returns properties of a given object.

    Object group of the result is inherited from the target object. # noqa
    """
    ...


async def global_lexical_scope_names() -> None:
    """Returns all let, const and class variables from global scope.

    # noqa
    """
    ...


async def query_objects() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def release_object() -> None:
    """Releases remote object with given id.

    # noqa
    """
    ...


async def release_object_group() -> None:
    """Releases all remote objects that belong to a given group.

    # noqa
    """
    ...


async def run_if_waiting_for_debugger() -> None:
    """Tells inspected instance to run if it was waiting for debugger to attach.

    # noqa
    """
    ...


async def run_script() -> None:
    """Runs script with given id in a given context.

    # noqa
    """
    ...


async def set_async_call_stack_depth() -> None:
    """Enables or disables async call stacks tracking.

    # noqa
    """
    ...


async def set_custom_object_formatter_enabled() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def set_max_call_stack_size_to_capture() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def terminate_execution() -> None:
    """Terminate current or next JavaScript execution.

    Will cancel the termination when the outer-most script execution ends. # noqa
    """
    ...


async def add_binding() -> None:
    """If executionContextId is empty, adds binding with the given name on the global objects of all inspected contexts,
    including those created later, bindings survive reloads.

    Binding function takes exactly one argument, this argument should be string, in case of any other input, function
    throws an exception. Each binding function call produces Runtime.bindingCalled notification. # noqa
    """
    ...


async def remove_binding() -> None:
    """This method does not remove binding function from global object but unsubscribes current runtime agent from
    Runtime.bindingCalled notifications.

    # noqa
    """
    ...


async def get_exception_details() -> None:
    """This method tries to lookup and populate exception details for a JavaScript Error object.

    Note that the stackTrace portion of the resulting exceptionDetails will only be populated if the Runtime domain was
    enabled at the time when the Error was thrown. # noqa
    """
    ...



# THIS FILE HAS BEEN AUTOMATICALLY GENERATED.
# CHANGES TO THIS FILE ARE FUTILE AS IT WILL BE OVERWRITTEN
# WHEN THE DEVTOOLS PROTOCOL CHANGES.  IF THERE ANY BUGS
# OR YOU WISH TO CHANGE HOW THE FILES ARE GENERATED PLEASE
# REFER TO: https://github.com/symonk/python-cdp OR YOUR
# OWN FORK.  REFERENCE THE `generate.py` FILE FOR CONTEXT
# AND INSTRUCTIONS.
# Chrome Devtools Protocol Domain Mapped to: `Console`.
# Url for domain: https://chromedevtools.github.io/devtools-protocol/tot/Console/

from __future__ import annotations

import typing
from dataclasses import dataclass

from .utils import memoize_event


@dataclass
class ConsoleMessage:
    """Console message."""

    # Message source. # noqa
    source: typing.List[
        typing.Literal[
            "xml",
            "javascript",
            "network",
            "console-api",
            "storage",
            "appcache",
            "rendering",
            "security",
            "other",
            "deprecation",
            "worker",
        ]
    ]
    # Message severity. # noqa
    level: typing.List[typing.Literal["log", "warning", "error", "debug", "info"]]
    # Message text. # noqa
    text: str
    # URL of the message origin. # noqa
    url: typing.Optional[str]
    # Line number in the resource that generated this message (1-based). # noqa
    line: typing.Optional[int]
    # Column number in the resource that generated this message (1-based). # noqa
    column: typing.Optional[int]


@dataclass
@memoize_event("Console.messageAdded")
class MessageAdded:
    """Issued when new console message is added."""

    message: ConsoleMessage


async def clear_messages() -> None:
    """Does nothing.

    # noqa
    """
    ...


async def disable() -> None:
    """Disables console domain, prevents further console messages from being reported to the client.

    # noqa
    """
    ...


async def enable() -> None:
    """Enables console domain, sends the messages collected so far to the client by means of the `messageAdded`
    notification.

    # noqa
    """
    ...



"""The module responsible for parsing chrome devtools protocol specification files into python objects."""
import functools
import operator
import typing

import pkg_resources
from loguru import logger

from python_cdp._utils import parse_browser_specification
from python_cdp._utils import parse_javascript_specification

from ._commandline import Configuration
from ._commandline import build_configuration
from ._models import TopLevelDomains

VERSION = pkg_resources.get_distribution("python-cdp")


def generate() -> int:
    """The main entrypoint to the script.

    This function is responsible for reading the whole protocol from the submodule and recursively generating all the
    source code in the /cdp directory.
    """
    config = initialise()  # noqa
    browser_spec = patch_protocol(parse_browser_specification())
    javascript_spec = patch_protocol(parse_javascript_specification())
    major, minor = operator.itemgetter("major", "minor")(browser_spec["version"])
    logger.info(f"Generating CDP code for version({major}.{minor})")
    generate_from_spec(browser_spec)
    generate_from_spec(javascript_spec)
    return 0


def initialise() -> Configuration:
    """Initialises the generation scrtip."""
    logger.info(f"python-cdp generation ({VERSION}) initialised.")
    return build_configuration()


def generate_from_spec(spec) -> None:
    """Generates the files for each spec."""
    for domain in TopLevelDomains.from_json(spec):
        logger.info(f"📖 Parsing {domain.domain} Devtools Protocol Module.")
        domain.create_py_module()


def patch_protocol(spec) -> typing.Dict[str, typing.Any]:
    """The protocol has a bunch of bugs!

    This method attempts to patch alot of them until I raise and fix
    issues in the upstream repository / protocol.

    This method patches the JSON blob in memory at runtime.
    """
    for domain in spec["domains"]:
        name, dependencies = domain["domain"], domain.get("dependencies", [])
        domain["dependencies"] = dependencies  # case where dependencies was missing!
        fn = functools.partial(patch_dependency, name, dependencies)
        if name == "Audits":
            fn(set(), {"DOM", "Page", "Runtime"})
        elif name == "BackgroundService":
            fn(set(), {"Network", "ServiceWorker"})
        elif name == "DOM":
            fn(set(), {"Page"})
        elif name == "Accessibility":
            fn(set(), {"Page"})
        elif name == "Target":
            fn(set(), {"Browser", "Page"})
        elif name == "Security":
            fn(set(), {"Network"})
        elif name == "Preload":
            fn(set(), {"Network", "DOM"})
        elif name == "Network":
            fn(set(), {"IO"})
        elif name == "PerformanceTimeline":
            fn(set(), {"Page"})
    return spec


def patch_dependency(name: str, dependencies: typing.List[str], remove: typing.Set[str], add: typing.Set[str]) -> None:
    """Patch dependencies for a given domain in place."""
    logger.info(f"Patching protocol specification for {name=}.  Original dependencies: {dependencies}")
    for element in remove:
        dependencies.remove(element)
    for element in add:
        dependencies.append(element)
    logger.info(f"Post patching dependencies: {dependencies}")


if __name__ == "__main__":
    raise SystemExit(generate())



# THIS FILE HAS BEEN AUTOMATICALLY GENERATED.
# CHANGES TO THIS FILE ARE FUTILE AS IT WILL BE OVERWRITTEN
# WHEN THE DEVTOOLS PROTOCOL CHANGES.  IF THERE ANY BUGS
# OR YOU WISH TO CHANGE HOW THE FILES ARE GENERATED PLEASE
# REFER TO: https://github.com/symonk/python-cdp OR YOUR
# OWN FORK.  REFERENCE THE `generate.py` FILE FOR CONTEXT
# AND INSTRUCTIONS.
# Chrome Devtools Protocol Domain Mapped to: `Debugger`.
# Url for domain: https://chromedevtools.github.io/devtools-protocol/tot/Debugger/

from __future__ import annotations

import enum
import typing
from dataclasses import dataclass

from . import runtime
from .utils import memoize_event


class BreakpointId(str):
    """Breakpoint identifier."""

    def to_json(self) -> BreakpointId:
        return self

    @classmethod
    def from_json(cls, value: str) -> BreakpointId:
        return cls(value)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(({super().__repr__()}))"


class CallFrameId(str):
    """Call frame identifier."""

    def to_json(self) -> CallFrameId:
        return self

    @classmethod
    def from_json(cls, value: str) -> CallFrameId:
        return cls(value)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(({super().__repr__()}))"


@dataclass
class Location:
    """Location in the source code."""

    # Script identifier as reported in the `Debugger.scriptParsed`. # noqa
    script_id: runtime.ScriptId
    # Line number in the script (0-based). # noqa
    line_number: int
    # Column number in the script (0-based). # noqa
    column_number: typing.Optional[int]


@dataclass
class ScriptPosition:
    """Location in the source code."""

    # Description is missing from the devtools protocol document. # noqa
    line_number: int
    # Description is missing from the devtools protocol document. # noqa
    column_number: int


@dataclass
class LocationRange:
    """Location range within one script."""

    # Description is missing from the devtools protocol document. # noqa
    script_id: runtime.ScriptId
    # Description is missing from the devtools protocol document. # noqa
    start: ScriptPosition
    # Description is missing from the devtools protocol document. # noqa
    end: ScriptPosition


@dataclass
class CallFrame:
    """JavaScript call frame.

    Array of call frames form the call stack.
    """

    # Call frame identifier. This identifier is only valid while the virtualmachine is paused. # noqa
    call_frame_id: CallFrameId
    # Name of the JavaScript function called on this call frame. # noqa
    function_name: str
    # Location in the source code. # noqa
    location: Location
    # JavaScript script name or url. Deprecated in favor of using the`location.scriptId` to resolve the URL via a previously sent`Debugger.scriptParsed` event. # noqa
    url: str
    # Scope chain for this call frame. # noqa
    scope_chain: Scope
    # `this` object for this call frame. # noqa
    this: runtime.RemoteObject
    # Location in the source code. # noqa
    function_location: typing.Optional[Location]
    # The value being returned, if the function is at return point. # noqa
    return_value: typing.Optional[runtime.RemoteObject]
    # Valid only while the VM is paused and indicates whether this frame can berestarted or not. Note that a `true` value here does not guarantee thatDebugger#restartFrame with this CallFrameId will be successful, but it is verylikely. # noqa
    can_be_restarted: typing.Optional[bool]


@dataclass
class Scope:
    """Scope description."""

    # Scope type. # noqa
    type: typing.List[
        typing.Literal[
            "global",
            "local",
            "with",
            "closure",
            "catch",
            "block",
            "script",
            "eval",
            "module",
            "wasm-expression-stack",
        ]
    ]
    # Object representing the scope. For `global` and `with` scopes itrepresents the actual object; for the rest of the scopes, it is artificialtransient object enumerating scope variables as its properties. # noqa
    object: runtime.RemoteObject
    # Description is missing from the devtools protocol document. # noqa
    name: typing.Optional[str]
    # Location in the source code where scope starts # noqa
    start_location: typing.Optional[Location]
    # Location in the source code where scope ends # noqa
    end_location: typing.Optional[Location]


@dataclass
class SearchMatch:
    """Search match for resource."""

    # Line number in resource content. # noqa
    line_number: float
    # Line with match content. # noqa
    line_content: str


@dataclass
class BreakLocation:
    """Description is missing from the devtools protocol document."""

    # Script identifier as reported in the `Debugger.scriptParsed`. # noqa
    script_id: runtime.ScriptId
    # Line number in the script (0-based). # noqa
    line_number: int
    # Column number in the script (0-based). # noqa
    column_number: typing.Optional[int]
    # Description is missing from the devtools protocol document. # noqa
    type: typing.Optional[typing.List[typing.Literal["debuggerStatement", "call", "return"]]]


@dataclass
class WasmDisassemblyChunk:
    """Description is missing from the devtools protocol document."""

    # The next chunk of disassembled lines. # noqa
    lines: str
    # The bytecode offsets describing the start of each line. # noqa
    bytecode_offsets: int


class ScriptLanguage(str, enum.Enum):
    """Enum of possible script languages."""

    JAVA_SCRIPT = "java_script"
    WEB_ASSEMBLY = "web_assembly"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


@dataclass
class DebugSymbols:
    """Debug symbols available for a wasm script."""

    # Type of the debug symbols. # noqa
    type: typing.List[typing.Literal["None", "SourceMap", "EmbeddedDWARF", "ExternalDWARF"]]
    # URL of the external symbol source. # noqa
    external_url: typing.Optional[str]


@dataclass
@memoize_event("Debugger.breakpointResolved")
class BreakpointResolved:
    """Fired when breakpoint is resolved to an actual script and location."""

    breakpoint_id: BreakpointId
    location: Location


@dataclass
@memoize_event("Debugger.paused")
class Paused:
    """Fired when the virtual machine stopped on breakpoint or exception or any other stop criteria."""

    call_frames: typing.List[CallFrame]
    reason: typing.Literal[
        "ambiguous",
        "assert",
        "csp_violation",
        "debug_command",
        "dom",
        "event_listener",
        "exception",
        "instrumentation",
        "oom",
        "other",
        "promise_rejection",
        "xhr",
        "step",
    ]
    data: typing.Optional[typing.Any]
    hit_breakpoints: typing.Optional[typing.List[str]]
    async_stack_trace: typing.Optional[runtime.StackTrace]
    async_stack_trace_id: typing.Optional[runtime.StackTraceId]
    async_call_stack_trace_id: typing.Optional[runtime.StackTraceId]


@dataclass
@memoize_event("Debugger.resumed")
class Resumed:
    """Fired when the virtual machine resumed execution."""


@dataclass
@memoize_event("Debugger.scriptFailedToParse")
class ScriptFailedToParse:
    """Fired when virtual machine fails to parse the script."""

    script_id: runtime.ScriptId
    url: str
    start_line: int
    start_column: int
    end_line: int
    end_column: int
    execution_context_id: runtime.ExecutionContextId
    hash: str
    execution_context_aux_data: typing.Optional[typing.Any]
    source_map_url: typing.Optional[str]
    has_source_url: typing.Optional[bool]
    is_module: typing.Optional[bool]
    length: typing.Optional[int]
    stack_trace: typing.Optional[runtime.StackTrace]
    code_offset: typing.Optional[int]
    script_language: typing.Optional[ScriptLanguage]
    embedder_name: typing.Optional[str]


@dataclass
@memoize_event("Debugger.scriptParsed")
class ScriptParsed:
    """Fired when virtual machine parses script.

    This event is also fired for all known and uncollected scripts upon enabling debugger.
    """

    script_id: runtime.ScriptId
    url: str
    start_line: int
    start_column: int
    end_line: int
    end_column: int
    execution_context_id: runtime.ExecutionContextId
    hash: str
    execution_context_aux_data: typing.Optional[typing.Any]
    is_live_edit: typing.Optional[bool]
    source_map_url: typing.Optional[str]
    has_source_url: typing.Optional[bool]
    is_module: typing.Optional[bool]
    length: typing.Optional[int]
    stack_trace: typing.Optional[runtime.StackTrace]
    code_offset: typing.Optional[int]
    script_language: typing.Optional[ScriptLanguage]
    debug_symbols: typing.Optional[DebugSymbols]
    embedder_name: typing.Optional[str]


async def continue_to_location() -> None:
    """Continues execution until specific location is reached.

    # noqa
    """
    ...


async def disable() -> None:
    """Disables debugger for given page.

    # noqa
    """
    ...


async def enable() -> None:
    """Enables debugger for the given page.

    Clients should not assume that the debugging has been enabled until the result for this command is received. # noqa
    """
    ...


async def evaluate_on_call_frame() -> None:
    """Evaluates expression on a given call frame.

    # noqa
    """
    ...


async def get_possible_breakpoints() -> None:
    """Returns possible locations for breakpoint.

    scriptId in start and end range locations should be the same. # noqa
    """
    ...


async def get_script_source() -> None:
    """Returns source for the script with given id.

    # noqa
    """
    ...


async def disassemble_wasm_module() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def next_wasm_disassembly_chunk() -> None:
    """Disassemble the next chunk of lines for the module corresponding to the stream.

    If disassembly is complete, this API will invalidate the streamId and return an empty chunk. Any subsequent calls
    for the now invalid stream will return errors. # noqa
    """
    ...


async def get_wasm_bytecode() -> None:
    """This command is deprecated.

    Use getScriptSource instead. # noqa
    """
    ...


async def get_stack_trace() -> None:
    """Returns stack trace with given `stackTraceId`.

    # noqa
    """
    ...


async def pause() -> None:
    """Stops on the next JavaScript statement.

    # noqa
    """
    ...


async def pause_on_async_call() -> None:
    """Description is missing from the devtools protocol document.

    # noqa
    """
    ...


async def remove_breakpoint() -> None:
    """Removes JavaScript breakpoint.

    # noqa
    """
    ...


async def restart_frame() -> None:
    """Restarts particular call frame from the beginning. The old, deprecated behavior of `restartFrame` is to stay
    paused and allow further CDP commands after a restart was scheduled. This can cause problems with restarting, so we
    now continue execution immediatly after it has been scheduled until we reach the beginning of the restarted frame.

    To stay back-wards compatible, `restartFrame` now expects a `mode`
    parameter to be present. If the `mode` parameter is missing, `restartFrame`
    errors out.

    The various return values are deprecated and `callFrames` is always empty.
    Use the call frames from the `Debugger#paused` events instead, that fires
    once V8 pauses at the beginning of the restarted function. # noqa
    """
    ...


async def resume() -> None:
    """Resumes JavaScript execution.

    # noqa
    """
    ...


async def search_in_content() -> None:
    """Searches for given string in script content.

    # noqa
    """
    ...


async def set_async_call_stack_depth() -> None:
    """Enables or disables async call stacks tracking.

    # noqa
    """
    ...


async def set_blackbox_patterns() -> None:
    """Replace previous blackbox patterns with passed ones.

    Forces backend to skip stepping/pausing in scripts with url matching one of the patterns. VM will try to leave
    blackboxed script by performing 'step in' several times, finally resorting to 'step out' if unsuccessful. # noqa
    """
    ...


async def set_blackboxed_ranges() -> None:
    """Makes backend skip steps in the script in blackboxed ranges.

    VM will try leave blacklisted scripts by performing 'step in' several times, finally resorting to 'step out' if
    unsuccessful. Positions array contains positions where blackbox state is changed. First interval isn't blackboxed.
    Array should be sorted. # noqa
    """
    ...


async def set_breakpoint() -> None:
    """Sets JavaScript breakpoint at a given location.

    # noqa
    """
    ...


async def set_instrumentation_breakpoint() -> None:
    """Sets instrumentation breakpoint.

    # noqa
    """
    ...


async def set_breakpoint_by_url() -> None:
    """Sets JavaScript breakpoint at given location specified either by URL or URL regex.

    Once this command is issued, all existing parsed scripts will have breakpoints resolved and returned in `locations`
    property. Further matching script parsing will result in subsequent `breakpointResolved` events issued. This logical
    breakpoint will survive page reloads. # noqa
    """
    ...


async def set_breakpoint_on_function_call() -> None:
    """Sets JavaScript breakpoint before each call to the given function.

    If another function was created from the same source as a given one, calling it will also trigger the breakpoint. #
    noqa
    """
    ...


async def set_breakpoints_active() -> None:
    """Activates / deactivates all breakpoints on the page.

    # noqa
    """
    ...


async def set_pause_on_exceptions() -> None:
    """Defines pause on exceptions state.

    Can be set to stop on all exceptions, uncaught exceptions, or caught exceptions, no exceptions. Initial pause on
    exceptions state is `none`. # noqa
    """
    ...


async def set_return_value() -> None:
    """Changes return value in top frame.

    Available only at return break position. # noqa
    """
    ...


async def set_script_source() -> None:
    """Edits JavaScript source live.

    In general, functions that are currently on the stack can not be edited with a single exception: If the edited
    function is the top-most stack frame and that is the only activation of that function on the stack. In this case the
    live edit will be successful and a `Debugger.restartFrame` for the top-most function is automatically triggered. #
    noqa
    """
    ...


async def set_skip_all_pauses() -> None:
    """Makes page not interrupt on any pauses (breakpoint, exception, dom exception etc).

    # noqa
    """
    ...


async def set_variable_value() -> None:
    """Changes value of variable in a callframe.

    Object-based scopes are not supported and must be mutated manually. # noqa
    """
    ...


async def step_into() -> None:
    """Steps into the function call.

    # noqa
    """
    ...


async def step_out() -> None:
    """Steps out of the function call.

    # noqa
    """
    ...


async def step_over() -> None:
    """Steps over the statement.

    # noqa
    """
    ...



# THIS FILE HAS BEEN AUTOMATICALLY GENERATED.
# CHANGES TO THIS FILE ARE FUTILE AS IT WILL BE OVERWRITTEN
# WHEN THE DEVTOOLS PROTOCOL CHANGES.  IF THERE ANY BUGS
# OR YOU WISH TO CHANGE HOW THE FILES ARE GENERATED PLEASE
# REFER TO: https://github.com/symonk/python-cdp OR YOUR
# OWN FORK.  REFERENCE THE `generate.py` FILE FOR CONTEXT
# AND INSTRUCTIONS.
# Chrome Devtools Protocol Domain Mapped to: `Accessibility`.
# Url for domain: https://chromedevtools.github.io/devtools-protocol/tot/Accessibility/

from __future__ import annotations

import enum
import typing
from dataclasses import dataclass

from . import dom
from . import page
from .utils import memoize_event


class AXNodeId(str):
    """Unique accessibility node identifier."""

    def to_json(self) -> AXNodeId:
        return self

    @classmethod
    def from_json(cls, value: str) -> AXNodeId:
        return cls(value)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(({super().__repr__()}))"


class AXValueType(str, enum.Enum):
    """Enum of possible property types."""

    BOOLEAN = "boolean"
    TRISTATE = "tristate"
    BOOLEAN_OR_UNDEFINED = "boolean_or_undefined"
    IDREF = "idref"
    IDREF_LIST = "idref_list"
    INTEGER = "integer"
    NODE = "node"
    NODE_LIST = "node_list"
    NUMBER = "number"
    STRING = "string"
    COMPUTED_STRING = "computed_string"
    TOKEN = "token"
    TOKEN_LIST = "token_list"
    DOM_RELATION = "dom_relation"
    ROLE = "role"
    INTERNAL_ROLE = "internal_role"
    VALUE_UNDEFINED = "value_undefined"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


class AXValueSourceType(str, enum.Enum):
    """Enum of possible property sources."""

    ATTRIBUTE = "attribute"
    IMPLICIT = "implicit"
    STYLE = "style"
    CONTENTS = "contents"
    PLACEHOLDER = "placeholder"
    RELATED_ELEMENT = "related_element"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


class AXValueNativeSourceType(str, enum.Enum):
    """Enum of possible native property sources (as a subtype of a particular AXValueSourceType)."""

    DESCRIPTION = "description"
    FIGCAPTION = "figcaption"
    LABEL = "label"
    LABELFOR = "labelfor"
    LABELWRAPPED = "labelwrapped"
    LEGEND = "legend"
    RUBYANNOTATION = "rubyannotation"
    TABLECAPTION = "tablecaption"
    TITLE = "title"
    OTHER = "other"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


@dataclass
class AXValueSource:
    """A single source for a computed AX property."""

    # What type of source this is. # noqa
    type: AXValueSourceType
    # The value of this property source. # noqa
    value: typing.Optional[AXValue]
    # The name of the relevant attribute, if any. # noqa
    attribute: typing.Optional[str]
    # The value of the relevant attribute, if any. # noqa
    attribute_value: typing.Optional[AXValue]
    # Whether this source is superseded by a higher priority source. # noqa
    superseded: typing.Optional[bool]
    # The native markup source for this value, e.g. a <label> element. # noqa
    native_source: typing.Optional[AXValueNativeSourceType]
    # The value, such as a node or node list, of the native source. # noqa
    native_source_value: typing.Optional[AXValue]
    # Whether the value for this property is invalid. # noqa
    invalid: typing.Optional[bool]
    # Reason for the value being invalid, if it is. # noqa
    invalid_reason: typing.Optional[str]


@dataclass
class AXRelatedNode:
    """Description is missing from the devtools protocol document."""

    # The BackendNodeId of the related DOM node. # noqa
    backend_dom_node_id: dom.BackendNodeId
    # The IDRef value provided, if any. # noqa
    idref: typing.Optional[str]
    # The text alternative of this node in the current context. # noqa
    text: typing.Optional[str]


@dataclass
class AXProperty:
    """Description is missing from the devtools protocol document."""

    # The name of this property. # noqa
    name: AXPropertyName
    # The value of this property. # noqa
    value: AXValue


@dataclass
class AXValue:
    """A single computed AX property."""

    # The type of this value. # noqa
    type: AXValueType
    # The computed value of this property. # noqa
    value: typing.Any
    # One or more related nodes, if applicable. # noqa
    related_nodes: typing.Optional[AXRelatedNode]
    # The sources which contributed to the computation of this property. # noqa
    sources: typing.Optional[AXValueSource]


class AXPropertyName(str, enum.Enum):
    """Values of AXProperty name:

    - from 'busy' to 'roledescription': states which apply to every AX node
    - from 'live' to 'root': attributes which apply to nodes in live regions
    - from 'autocomplete' to 'valuetext': attributes which apply to widgets
    - from 'checked' to 'selected': states which apply to widgets
    - from 'activedescendant' to 'owns' - relationships between elements other than parent/child/sibling.
    """

    BUSY = "busy"
    DISABLED = "disabled"
    EDITABLE = "editable"
    FOCUSABLE = "focusable"
    FOCUSED = "focused"
    HIDDEN = "hidden"
    HIDDEN_ROOT = "hidden_root"
    INVALID = "invalid"
    KEYSHORTCUTS = "keyshortcuts"
    SETTABLE = "settable"
    ROLEDESCRIPTION = "roledescription"
    LIVE = "live"
    ATOMIC = "atomic"
    RELEVANT = "relevant"
    ROOT = "root"
    AUTOCOMPLETE = "autocomplete"
    HAS_POPUP = "has_popup"
    LEVEL = "level"
    MULTISELECTABLE = "multiselectable"
    ORIENTATION = "orientation"
    MULTILINE = "multiline"
    READONLY = "readonly"
    REQUIRED = "required"
    VALUEMIN = "valuemin"
    VALUEMAX = "valuemax"
    VALUETEXT = "valuetext"
    CHECKED = "checked"
    EXPANDED = "expanded"
    MODAL = "modal"
    PRESSED = "pressed"
    SELECTED = "selected"
    ACTIVEDESCENDANT = "activedescendant"
    CONTROLS = "controls"
    DESCRIBEDBY = "describedby"
    DETAILS = "details"
    ERRORMESSAGE = "errormessage"
    FLOWTO = "flowto"
    LABELLEDBY = "labelledby"
    OWNS = "owns"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


@dataclass
class AXNode:
    """A node in the accessibility tree."""

    # Unique identifier for this node. # noqa
    node_id: AXNodeId
    # Whether this node is ignored for accessibility # noqa
    ignored: bool
    # Collection of reasons why this node is hidden. # noqa
    ignored_reasons: typing.Optional[AXProperty]
    # This `Node`'s role, whether explicit or implicit. # noqa
    role: typing.Optional[AXValue]
    # This `Node`'s Chrome raw role. # noqa
    chrome_role: typing.Optional[AXValue]
    # The accessible name for this `Node`. # noqa
    name: typing.Optional[AXValue]
    # The accessible description for this `Node`. # noqa
    description: typing.Optional[AXValue]
    # The value for this `Node`. # noqa
    value: typing.Optional[AXValue]
    # All other properties # noqa
    properties: typing.Optional[AXProperty]
    # ID for this node's parent. # noqa
    parent_id: typing.Optional[AXNodeId]
    # IDs for each of this node's child nodes. # noqa
    child_ids: typing.Optional[AXNodeId]
    # The backend ID for the associated DOM node, if any. # noqa
    backend_dom_node_id: typing.Optional[dom.BackendNodeId]
    # The frame ID for the frame associated with this nodes document. # noqa
    frame_id: typing.Optional[page.FrameId]


@dataclass
@memoize_event("Accessibility.loadComplete")
class LoadComplete:
    """The loadComplete event mirrors the load complete event sent by the browser to assistive technology when the web
    page has finished loading."""

    root: AXNode


@dataclass
@memoize_event("Accessibility.nodesUpdated")
class NodesUpdated:
    """The nodesUpdated event is sent every time a previously requested node has changed the in tree."""

    nodes: typing.List[AXNode]


async def disable() -> None:
    """Disables the accessibility domain.

    # noqa
    """
    ...


async def enable() -> None:
    """Enables the accessibility domain which causes `AXNodeId`s to remain consistent between method calls.

    This turns on accessibility for the page, which can impact performance until accessibility is disabled. # noqa
    """
    ...


async def get_partial_ax_tree() -> None:
    """Fetches the accessibility node and partial accessibility tree for this DOM node, if it exists.

    # noqa
    """
    ...


async def get_full_ax_tree() -> None:
    """Fetches the entire accessibility tree for the root Document # noqa."""
    ...


async def get_root_ax_node() -> None:
    """Fetches the root node.

    Requires `enable()` to have been called previously. # noqa
    """
    ...


async def get_ax_node_and_ancestors() -> None:
    """Fetches a node and all ancestors up to and including the root.

    Requires `enable()` to have been called previously. # noqa
    """
    ...


async def get_child_ax_nodes() -> None:
    """Fetches a particular accessibility node by AXNodeId.

    Requires `enable()` to have been called previously. # noqa
    """
    ...


async def query_ax_tree() -> None:
    """Query a DOM node's accessibility subtree for accessible name and role.

    This command computes the name and role for all nodes in the subtree, including those that are ignored for
    accessibility, and returns those that mactch the specified name and role. If no DOM node is specified, or the DOM
    node does not exist, the command returns an error. If neither `accessibleName` or `role` is specified, it returns
    all the accessibility nodes in the subtree. # noqa
    """
    ...



# THIS FILE HAS BEEN AUTOMATICALLY GENERATED.
# CHANGES TO THIS FILE ARE FUTILE AS IT WILL BE OVERWRITTEN
# WHEN THE DEVTOOLS PROTOCOL CHANGES.  IF THERE ANY BUGS
# OR YOU WISH TO CHANGE HOW THE FILES ARE GENERATED PLEASE
# REFER TO: https://github.com/symonk/python-cdp OR YOUR
# OWN FORK.  REFERENCE THE `generate.py` FILE FOR CONTEXT
# AND INSTRUCTIONS.
# Chrome Devtools Protocol Domain Mapped to: `EventBreakpoints`.
# Url for domain: https://chromedevtools.github.io/devtools-protocol/tot/EventBreakpoints/

from __future__ import annotations


async def set_instrumentation_breakpoint() -> None:
    """Sets breakpoint on particular native event.

    # noqa
    """
    ...


async def remove_instrumentation_breakpoint() -> None:
    """Removes breakpoint on particular native event.

    # noqa
    """
    ...



# THIS FILE HAS BEEN AUTOMATICALLY GENERATED.
# CHANGES TO THIS FILE ARE FUTILE AS IT WILL BE OVERWRITTEN
# WHEN THE DEVTOOLS PROTOCOL CHANGES.  IF THERE ANY BUGS
# OR YOU WISH TO CHANGE HOW THE FILES ARE GENERATED PLEASE
# REFER TO: https://github.com/symonk/python-cdp OR YOUR
# OWN FORK.  REFERENCE THE `generate.py` FILE FOR CONTEXT
# AND INSTRUCTIONS.
# Chrome Devtools Protocol Domain Mapped to: `WebAuthn`.
# Url for domain: https://chromedevtools.github.io/devtools-protocol/tot/WebAuthn/

from __future__ import annotations

import enum
import typing
from dataclasses import dataclass

from .utils import memoize_event


class AuthenticatorId(str):
    """Description is missing from the devtools protocol document."""

    def to_json(self) -> AuthenticatorId:
        return self

    @classmethod
    def from_json(cls, value: str) -> AuthenticatorId:
        return cls(value)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(({super().__repr__()}))"


class AuthenticatorProtocol(str, enum.Enum):
    """Description is missing from the devtools protocol document."""

    U2F = "u2f"
    CTAP2 = "ctap2"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


class Ctap2Version(str, enum.Enum):
    """Description is missing from the devtools protocol document."""

    CTAP2_0 = "ctap2_0"
    CTAP2_1 = "ctap2_1"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


class AuthenticatorTransport(str, enum.Enum):
    """Description is missing from the devtools protocol document."""

    USB = "usb"
    NFC = "nfc"
    BLE = "ble"
    CABLE = "cable"
    INTERNAL = "internal"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


@dataclass
class VirtualAuthenticatorOptions:
    """Description is missing from the devtools protocol document."""

    # Description is missing from the devtools protocol document. # noqa
    protocol: AuthenticatorProtocol
    # Description is missing from the devtools protocol document. # noqa
    transport: AuthenticatorTransport
    # Defaults to ctap2_0. Ignored if |protocol| == u2f. # noqa
    ctap2_version: typing.Optional[Ctap2Version]
    # Defaults to false. # noqa
    has_resident_key: typing.Optional[bool]
    # Defaults to false. # noqa
    has_user_verification: typing.Optional[bool]
    # If set to true, the authenticator will support the largeBlob extension.https://w3c.github.io/webauthn#largeBlob Defaults to false. # noqa
    has_large_blob: typing.Optional[bool]
    # If set to true, the authenticator will support the credBlob extension.https://fidoalliance.org/specs/fido-v2.1-rd-20201208/fido-client-to-authenticator-protocol-v2.1-rd-20201208.html#sctn-credBlob-extension Defaults tofalse. # noqa
    has_cred_blob: typing.Optional[bool]
    # If set to true, the authenticator will support the minPinLength extension.https://fidoalliance.org/specs/fido-v2.1-ps-20210615/fido-client-to-authenticator-protocol-v2.1-ps-20210615.html#sctn-minpinlength-extensionDefaults to false. # noqa
    has_min_pin_length: typing.Optional[bool]
    # If set to true, the authenticator will support the prf extension.https://w3c.github.io/webauthn/#prf-extension Defaults to false. # noqa
    has_prf: typing.Optional[bool]
    # If set to true, tests of user presence will succeed immediately.Otherwise, they will not be resolved. Defaults to true. # noqa
    automatic_presence_simulation: typing.Optional[bool]
    # Sets whether User Verification succeeds or fails for an authenticator.Defaults to false. # noqa
    is_user_verified: typing.Optional[bool]


@dataclass
class Credential:
    """Description is missing from the devtools protocol document."""

    # Description is missing from the devtools protocol document. # noqa
    credential_id: str
    # Description is missing from the devtools protocol document. # noqa
    is_resident_credential: bool
    # The ECDSA P-256 private key in PKCS#8 format. (Encoded as a base64 stringwhen passed over JSON) # noqa
    private_key: str
    # Signature counter. This is incremented by one for each successfulassertion. See https://w3c.github.io/webauthn/#signature-counter # noqa
    sign_count: int
    # Relying Party ID the credential is scoped to. Must be set when adding acredential. # noqa
    rp_id: typing.Optional[str]
    # An opaque byte sequence with a maximum size of 64 bytes mapping thecredential to a specific user. (Encoded as a base64 string when passed overJSON) # noqa
    user_handle: typing.Optional[str]
    # The large blob associated with the credential. Seehttps://w3c.github.io/webauthn/#sctn-large-blob-extension (Encoded as a base64string when passed over JSON) # noqa
    large_blob: typing.Optional[str]


@dataclass
@memoize_event("WebAuthn.credentialAdded")
class CredentialAdded:
    """Triggered when a credential is added to an authenticator."""

    authenticator_id: AuthenticatorId
    credential: Credential


@dataclass
@memoize_event("WebAuthn.credentialAsserted")
class CredentialAsserted:
    """Triggered when a credential is used in a webauthn assertion."""

    authenticator_id: AuthenticatorId
    credential: Credential


async def enable() -> None:
    """Enable the WebAuthn domain and start intercepting credential storage and retrieval with a virtual authenticator.

    # noqa
    """
    ...


async def disable() -> None:
    """Disable the WebAuthn domain.

    # noqa
    """
    ...


async def add_virtual_authenticator() -> None:
    """Creates and adds a virtual authenticator.

    # noqa
    """
    ...


async def set_response_override_bits() -> None:
    """Resets parameters isBogusSignature, isBadUV, isBadUP to false if they are not present.

    # noqa
    """
    ...


async def remove_virtual_authenticator() -> None:
    """Removes the given authenticator.

    # noqa
    """
    ...


async def add_credential() -> None:
    """Adds the credential to the specified authenticator.

    # noqa
    """
    ...


async def get_credential() -> None:
    """Returns a single credential stored in the given virtual authenticator that matches the credential ID.

    # noqa
    """
    ...


async def get_credentials() -> None:
    """Returns all the credentials stored in the given virtual authenticator.

    # noqa
    """
    ...


async def remove_credential() -> None:
    """Removes a credential from the authenticator.

    # noqa
    """
    ...


async def clear_credentials() -> None:
    """Clears all the credentials from the specified device.

    # noqa
    """
    ...


async def set_user_verified() -> None:
    """Sets whether User Verification succeeds or fails for an authenticator.

    The default is true. # noqa
    """
    ...


async def set_automatic_presence_simulation() -> None:
    """Sets whether tests of user presence will succeed immediately (if true) or fail to resolve (if false) for an
    authenticator.

    The default is true. # noqa
    """
    ...



# THIS FILE HAS BEEN AUTOMATICALLY GENERATED.
# CHANGES TO THIS FILE ARE FUTILE AS IT WILL BE OVERWRITTEN
# WHEN THE DEVTOOLS PROTOCOL CHANGES.  IF THERE ANY BUGS
# OR YOU WISH TO CHANGE HOW THE FILES ARE GENERATED PLEASE
# REFER TO: https://github.com/symonk/python-cdp OR YOUR
# OWN FORK.  REFERENCE THE `generate.py` FILE FOR CONTEXT
# AND INSTRUCTIONS.
# Chrome Devtools Protocol Domain Mapped to: `Security`.
# Url for domain: https://chromedevtools.github.io/devtools-protocol/tot/Security/

from __future__ import annotations

import enum
import typing
from dataclasses import dataclass

from . import network
from .utils import memoize_event


class CertificateId(int):
    """An internal certificate ID value."""

    def to_json(self) -> CertificateId:
        return self

    @classmethod
    def from_json(cls, value: int) -> CertificateId:
        return cls(value)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(({super().__repr__()}))"


class MixedContentType(str, enum.Enum):
    """A description of mixed content (HTTP resources on HTTPS pages), as defined by https://www.w3.org/TR/mixed-
    content/#categories."""

    BLOCKABLE = "blockable"
    OPTIONALLY_BLOCKABLE = "optionally-blockable"
    NONE = "none"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


class SecurityState(str, enum.Enum):
    """The security level of a page or resource."""

    UNKNOWN = "unknown"
    NEUTRAL = "neutral"
    INSECURE = "insecure"
    SECURE = "secure"
    INFO = "info"
    INSECURE_BROKEN = "insecure-broken"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


@dataclass
class CertificateSecurityState:
    """Details about the security state of the page certificate."""

    # Protocol name (e.g. "TLS 1.2" or "QUIC"). # noqa
    protocol: str
    # Key Exchange used by the connection, or the empty string if notapplicable. # noqa
    key_exchange: str
    # Cipher name. # noqa
    cipher: str
    # Page certificate. # noqa
    certificate: str
    # Certificate subject name. # noqa
    subject_name: str
    # Name of the issuing CA. # noqa
    issuer: str
    # Certificate valid from date. # noqa
    valid_from: network.TimeSinceEpoch
    # Certificate valid to (expiration) date # noqa
    valid_to: network.TimeSinceEpoch
    # True if the certificate uses a weak signature aglorithm. # noqa
    certificate_has_weak_signature: bool
    # True if the certificate has a SHA1 signature in the chain. # noqa
    certificate_has_sha1_signature: bool
    # True if modern SSL # noqa
    modern_ssl: bool
    # True if the connection is using an obsolete SSL protocol. # noqa
    obsolete_ssl_protocol: bool
    # True if the connection is using an obsolete SSL key exchange. # noqa
    obsolete_ssl_key_exchange: bool
    # True if the connection is using an obsolete SSL cipher. # noqa
    obsolete_ssl_cipher: bool
    # True if the connection is using an obsolete SSL signature. # noqa
    obsolete_ssl_signature: bool
    # (EC)DH group used by the connection, if applicable. # noqa
    key_exchange_group: typing.Optional[str]
    # TLS MAC. Note that AEAD ciphers do not have separate MACs. # noqa
    mac: typing.Optional[str]
    # The highest priority network error code, if the certificate has an error. # noqa
    certificate_network_error: typing.Optional[str]


class SafetyTipStatus(str, enum.Enum):
    """Description is missing from the devtools protocol document."""

    BAD_REPUTATION = "bad_reputation"
    LOOKALIKE = "lookalike"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


@dataclass
class SafetyTipInfo:
    """Description is missing from the devtools protocol document."""

    # Describes whether the page triggers any safety tips or reputationwarnings. Default is unknown. # noqa
    safety_tip_status: SafetyTipStatus
    # The URL the safety tip suggested ("Did you mean?"). Only filled in forlookalike matches. # noqa
    safe_url: typing.Optional[str]


@dataclass
class VisibleSecurityState:
    """Security state information about the page."""

    # The security level of the page. # noqa
    security_state: SecurityState
    # Array of security state issues ids. # noqa
    security_state_issue_ids: str
    # Security state details about the page certificate. # noqa
    certificate_security_state: typing.Optional[CertificateSecurityState]
    # The type of Safety Tip triggered on the page. Note that this field will beset even if the Safety Tip UI was not actually shown. # noqa
    safety_tip_info: typing.Optional[SafetyTipInfo]


@dataclass
class SecurityStateExplanation:
    """An explanation of an factor contributing to the security state."""

    # Security state representing the severity of the factor being explained. # noqa
    security_state: SecurityState
    # Title describing the type of factor. # noqa
    title: str
    # Short phrase describing the type of factor. # noqa
    summary: str
    # Full text explanation of the factor. # noqa
    description: str
    # The type of mixed content described by the explanation. # noqa
    mixed_content_type: MixedContentType
    # Page certificate. # noqa
    certificate: str
    # Recommendations to fix any issues. # noqa
    recommendations: typing.Optional[str]


@dataclass
class InsecureContentStatus:
    """Information about insecure content on the page."""

    # Always false. # noqa
    ran_mixed_content: bool
    # Always false. # noqa
    displayed_mixed_content: bool
    # Always false. # noqa
    contained_mixed_form: bool
    # Always false. # noqa
    ran_content_with_cert_errors: bool
    # Always false. # noqa
    displayed_content_with_cert_errors: bool
    # Always set to unknown. # noqa
    ran_insecure_content_style: SecurityState
    # Always set to unknown. # noqa
    displayed_insecure_content_style: SecurityState


class CertificateErrorAction(str, enum.Enum):
    """The action to take when a certificate error occurs.

    continue will continue processing the request and cancel will cancel the request.
    """

    CONTINUE = "continue"
    CANCEL = "cancel"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


@dataclass
@memoize_event("Security.certificateError")
class CertificateError:
    """There is a certificate error.

    If overriding certificate errors is enabled, then it should be handled with the `handleCertificateError` command.
    Note: this event does not fire if the certificate error has been allowed internally. Only one client per target
    should override certificate errors at the same time.
    """

    event_id: int
    error_type: str
    request_url: str


@dataclass
@memoize_event("Security.visibleSecurityStateChanged")
class VisibleSecurityStateChanged:
    """The security state of the page changed."""

    visible_security_state: VisibleSecurityState


@dataclass
@memoize_event("Security.securityStateChanged")
class SecurityStateChanged:
    """The security state of the page changed.

    No longer being sent.
    """

    security_state: SecurityState
    scheme_is_cryptographic: bool
    explanations: typing.List[SecurityStateExplanation]
    insecure_content_status: InsecureContentStatus
    summary: typing.Optional[str]


async def disable() -> None:
    """Disables tracking security state changes.

    # noqa
    """
    ...


async def enable() -> None:
    """Enables tracking security state changes.

    # noqa
    """
    ...


async def set_ignore_certificate_errors() -> None:
    """Enable/disable whether all certificate errors should be ignored.

    # noqa
    """
    ...


async def handle_certificate_error() -> None:
    """Handles a certificate error that fired a certificateError event.

    # noqa
    """
    ...


async def set_override_certificate_errors() -> None:
    """Enable/disable overriding certificate errors.

    If enabled, all certificate error events need to be handled by the DevTools client and should be answered with
    `handleCertificateError` commands. # noqa
    """
    ...



import argparse
from dataclasses import dataclass


@dataclass
class Configuration:
    """Encapsulation of the generation runtime configuration."""


def build_configuration() -> Configuration:
    parser = argparse.ArgumentParser()
    namespace = parser.parse_args()
    return Configuration(**vars(namespace))



# THIS FILE HAS BEEN AUTOMATICALLY GENERATED.
# CHANGES TO THIS FILE ARE FUTILE AS IT WILL BE OVERWRITTEN
# WHEN THE DEVTOOLS PROTOCOL CHANGES.  IF THERE ANY BUGS
# OR YOU WISH TO CHANGE HOW THE FILES ARE GENERATED PLEASE
# REFER TO: https://github.com/symonk/python-cdp OR YOUR
# OWN FORK.  REFERENCE THE `generate.py` FILE FOR CONTEXT
# AND INSTRUCTIONS.
# Chrome Devtools Protocol Domain Mapped to: `Performance`.
# Url for domain: https://chromedevtools.github.io/devtools-protocol/tot/Performance/

from __future__ import annotations

import typing
from dataclasses import dataclass

from .utils import memoize_event


@dataclass
class Metric:
    """Run-time execution metric."""

    # Metric name. # noqa
    name: str
    # Metric value. # noqa
    value: float


@dataclass
@memoize_event("Performance.metrics")
class Metrics:
    """Current values of the metrics."""

    metrics: typing.List[Metric]
    title: str


async def disable() -> None:
    """Disable collecting and reporting metrics.

    # noqa
    """
    ...


async def enable() -> None:
    """Enable collecting and reporting metrics.

    # noqa
    """
    ...


async def set_time_domain() -> None:
    """Sets time domain to use for collecting and reporting duration metrics.

    Note that this must be called before enabling metrics collection. Calling this method while metrics collection is
    enabled returns an error. # noqa
    """
    ...


async def get_metrics() -> None:
    """Retrieve current values of run-time metrics.

    # noqa
    """
    ...



# THIS FILE HAS BEEN AUTOMATICALLY GENERATED.
# CHANGES TO THIS FILE ARE FUTILE AS IT WILL BE OVERWRITTEN
# WHEN THE DEVTOOLS PROTOCOL CHANGES.  IF THERE ANY BUGS
# OR YOU WISH TO CHANGE HOW THE FILES ARE GENERATED PLEASE
# REFER TO: https://github.com/symonk/python-cdp OR YOUR
# OWN FORK.  REFERENCE THE `generate.py` FILE FOR CONTEXT
# AND INSTRUCTIONS.
# Chrome Devtools Protocol Domain Mapped to: `Input`.
# Url for domain: https://chromedevtools.github.io/devtools-protocol/tot/Input/

from __future__ import annotations

import enum
import typing
from dataclasses import dataclass

from .utils import memoize_event


@dataclass
class TouchPoint:
    """Description is missing from the devtools protocol document."""

    # X coordinate of the event relative to the main frame's viewport in CSSpixels. # noqa
    x: float
    # Y coordinate of the event relative to the main frame's viewport in CSSpixels. 0 refers to the top of the viewport and Y increases as it proceedstowards the bottom of the viewport. # noqa
    y: float
    # X radius of the touch area (default: 1.0). # noqa
    radius_x: typing.Optional[float]
    # Y radius of the touch area (default: 1.0). # noqa
    radius_y: typing.Optional[float]
    # Rotation angle (default: 0.0). # noqa
    rotation_angle: typing.Optional[float]
    # Force (default: 1.0). # noqa
    force: typing.Optional[float]
    # The normalized tangential pressure, which has a range of [-1,1] (default:0). # noqa
    tangential_pressure: typing.Optional[float]
    # The plane angle between the Y-Z plane and the plane containing both thestylus axis and the Y axis, in degrees of the range [-90,90], a positive tiltXis to the right (default: 0) # noqa
    tilt_x: typing.Optional[int]
    # The plane angle between the X-Z plane and the plane containing both thestylus axis and the X axis, in degrees of the range [-90,90], a positive tiltYis towards the user (default: 0). # noqa
    tilt_y: typing.Optional[int]
    # The clockwise rotation of a pen stylus around its own major axis, indegrees in the range [0,359] (default: 0). # noqa
    twist: typing.Optional[int]
    # Identifier used to track touch sources between events, must be uniquewithin an event. # noqa
    id: typing.Optional[float]


class GestureSourceType(str, enum.Enum):
    """Description is missing from the devtools protocol document."""

    DEFAULT = "default"
    TOUCH = "touch"
    MOUSE = "mouse"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


class MouseButton(str, enum.Enum):
    """Description is missing from the devtools protocol document."""

    NONE = "none"
    LEFT = "left"
    MIDDLE = "middle"
    RIGHT = "right"
    BACK = "back"
    FORWARD = "forward"

    @classmethod
    def from_json(cls, value: str) -> str:
        return cls(value)


class TimeSinceEpoch(float):
    """UTC time in seconds, counted from January 1, 1970."""

    def to_json(self) -> TimeSinceEpoch:
        return self

    @classmethod
    def from_json(cls, value: float) -> TimeSinceEpoch:
        return cls(value)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(({super().__repr__()}))"


@dataclass
class DragDataItem:
    """Description is missing from the devtools protocol document."""

    # Mime type of the dragged data. # noqa
    mime_type: str
    # Depending of the value of `mimeType`, it contains the dragged link, text,HTML markup or any other data. # noqa
    data: str
    # Title associated with a link. Only valid when `mimeType` == "text/uri-list". # noqa
    title: typing.Optional[str]
    # Stores the base URL for the contained markup. Only valid when `mimeType`== "text/html". # noqa
    base_url: typing.Optional[str]


@dataclass
class DragData:
    """Description is missing from the devtools protocol document."""

    # Description is missing from the devtools protocol document. # noqa
    items: DragDataItem
    # Bit field representing allowed drag operations. Copy = 1, Link = 2, Move =16 # noqa
    drag_operations_mask: int
    # List of filenames that should be included when dropping # noqa
    files: typing.Optional[str]


@dataclass
@memoize_event("Input.dragIntercepted")
class DragIntercepted:
    """Emitted only when `Input.setInterceptDrags` is enabled.

    Use this data with `Input.dispatchDragEvent` to restore normal drag and drop behavior.
    """

    data: DragData


async def dispatch_drag_event() -> None:
    """Dispatches a drag event into the page.

    # noqa
    """
    ...


async def dispatch_key_event() -> None:
    """Dispatches a key event to the page.

    # noqa
    """
    ...


async def insert_text() -> None:
    """This method emulates inserting text that doesn't come from a key press, for example an emoji keyboard or an IME.

    # noqa
    """
    ...


async def ime_set_composition() -> None:
    """This method sets the current candidate text for ime.

    Use imeCommitComposition to commit the final text. Use imeSetComposition with empty string as text to cancel
    composition. # noqa
    """
    ...


async def dispatch_mouse_event() -> None:
    """Dispatches a mouse event to the page.

    # noqa
    """
    ...


async def dispatch_touch_event() -> None:
    """Dispatches a touch event to the page.

    # noqa
    """
    ...


async def emulate_touch_from_mouse_event() -> None:
    """Emulates touch event from the mouse event parameters.

    # noqa
    """
    ...


async def set_ignore_input_events() -> None:
    """Ignores input events (useful while auditing page).

    # noqa
    """
    ...


async def set_intercept_drags() -> None:
    """Prevents default drag and drop behavior and instead emits `Input.dragIntercepted` events.

    Drag and drop behavior can be directly controlled via `Input.dispatchDragEvent`. # noqa
    """
    ...


async def synthesize_pinch_gesture() -> None:
    """Synthesizes a pinch gesture over a time period by issuing appropriate touch events.

    # noqa
    """
    ...


async def synthesize_scroll_gesture() -> None:
    """Synthesizes a scroll gesture over a time period by issuing appropriate touch events.

    # noqa
    """
    ...


async def synthesize_tap_gesture() -> None:
    """Synthesizes a tap gesture over a time period by issuing appropriate touch events.

    # noqa
    """
    ...
